create or replace PACKAGE BODY INMO_MOD_ESTADISTICAS AS
  -- Consulta1: Dado un ao cualquiera, porcentaje de inmuebles a alquilar que dispona la empresa y que se alquilaron durante ese ao
  -- Tarea:Incrementa en uno el nmero de inmuebles a alquilar, en el ao indicado. Si no hay datos para ese ao se aade.
  -- Nota: Slo consideramos las altas, entendemos que para esta estadstica no interesa controlar las bajas
  PROCEDURE ALTA_DWH_INMUEBLES_ALQUILAR (inmo_anio IN VARCHAR2) AS
 
    n_any integer;
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado

	  -- Verificamos si existe estadsticas de ao indicado
	  BEGIN
		SELECT COUNT(anio) INTO n_any FROM DWH_INMUEBLES_ALQUILAR WHERE anio = inmo_anio ;

	  -- Si no existe el ao  se inserta
	  IF (n_any = 0) THEN
		INSERT INTO DWH_INMUEBLES_ALQUILAR (ID,ANIO,NUM_INMUEBLES_ALQUILAR,NUM_INMUEBLES_ALQUILADOS)
		  VALUES (estadistica_seq.nextval,inmo_anio,0,0);
	  END IF;
	  -- Actualiza el nmero de inmuebles a alquilar
	  UPDATE DWH_INMUEBLES_ALQUILAR SET NUM_INMUEBLES_ALQUILAR = NUM_INMUEBLES_ALQUILAR + 1 WHERE anio = inmo_anio;

	  INSERT INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nuevo Inmueble a alquilar]','','OK',USER);

	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nuevo dispositivo a alquilar]','',error_desc,USER);
	  COMMIT;
    END ALTA_DWH_INMUEBLES_ALQUILAR;


   -- Consulta1: Dado un ao cualquiera, porcentaje de inmuebles a alquilar que dispona la empresa y que se alquilaron durante ese ao
  -- Tarea:Incrementa en uno el nmero de inmuebles a alquilados, en el ao indicado. Si no hay datos para ese ao se aade.
  -- Nota: Slo consideramos las altas, entendemos que para esta estadstica no interesa controlar las bajas

  PROCEDURE ALTA_DWH_INMUEBLES_ALQUILADOS (inmo_anio IN VARCHAR2) AS

    n_any integer;
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado

	  -- Verificamos si existe estadsticas de ao indicado
	  BEGIN
		SELECT COUNT(anio) INTO n_any FROM DWH_INMUEBLES_ALQUILAR WHERE anio = inmo_anio;

	  -- Si no existe el ao, se inserta
	  IF (n_any = 0) THEN
		INSERT INTO DWH_INMUEBLES_ALQUILAR (ID,ANIO,NUM_INMUEBLES_ALQUILAR,NUM_INMUEBLES_ALQUILADOS)
		  VALUES (estadistica_seq.nextval,inmo_anio,0,0);
	  END IF;
	  -- Actualiza el nmero de inmuebles a alquilar
	  UPDATE DWH_INMUEBLES_ALQUILAR SET NUM_INMUEBLES_ALQUILADOS = NUM_INMUEBLES_ALQUILADOS + 1 WHERE anio = inmo_anio;

	  INSERT INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nuevo Inmueble  alquilado]','','OK',USER);

	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: No se ha podido alquilar el inmueble]','',error_desc,USER);
	  COMMIT;
  END ALTA_DWH_INMUEBLES_ALQUILADOS;

   -- Consulta4: Sede que en el ao en curso ha realizado el mayor nmero de transacciones, considerando como transaccin la venta o alquiler de un inmueble cualquiera.
    -- Tarea:Incrementa en uno el nmero de transacciones  para la sede y el ao indicado. 

   --Consulta 5: Comercial que ms visitas ha realizado en el ao en curso.
   -- Tarea: incrementa en uno el nmero de visitas para el empleado y la sede indicada. 


  PROCEDURE ALTA_DWH_TRANSACCIONES_VISITAS (inmo_anio IN varchar2,inmo_sede IN SEDES.CODIGO_SEDE%TYPE,inmo_empleado IN EMPLEADOS.CODIGO_EMPLEADO%TYPE) AS
    error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
    n_any integer;
    max_sede DWH_TRANSACCIONES_VISITAS.COD_SEDE%TYPE;
    max_empleado DWH_TRANSACCIONES_VISITAS.COD_EMPLEADO%TYPE;

  BEGIN
  -- Verificamos si existe estadsticas de ao indicado para la Consulta4


    IF inmo_sede is not null then
        SELECT COUNT(anio) INTO n_any FROM DWH_TRANSACCIONES_VISITAS WHERE anio = inmo_anio AND COD_SEDE=inmo_sede;

      -- Si no existe el ao para esa sede, se inserta
	     IF (n_any = 0) THEN
		      INSERT INTO DWH_TRANSACCIONES_VISITAS (ID,ANIO,COD_SEDE,NUM_ACT_VISITAS,NUM_TRANSACCIONES,TOTAL_TRAN,TOTAL_VISITAS)
		      VALUES (estadistica_seq.nextval,inmo_anio,inmo_sede,0,0,0,0);
	     END IF;

    -- Actualiza el nmero de TRANSACCIONES DE LA SEDE    
	     UPDATE DWH_TRANSACCIONES_VISITAS SET NUM_TRANSACCIONES = NUM_TRANSACCIONES + 1 WHERE (anio = inmo_anio) and (cod_sede =inmo_sede);
    end if;

     -- busqueda de la sede con mximo numero de transacciones. El campo cod_sede_max contendr TRUE para la sede con mayor nmero de transacciones
   DECLARE 
      CURSOR c_suma IS 
      SELECT SUM(NUM_TRANSACCIONES)  FROM DWH_TRANSACCIONES_VISITAS WHERE ANIO < = EXTRACT(YEAR FROM SYSDATE)  AND COD_SEDE = inmo_sede;
        
       vc_suma NUMBER;    
    
    CURSOR c_maximo is
      SELECT MAX(TOTAL_TRAN) FROM DWH_TRANSACCIONES_VISITAS WHERE COD_SEDE IS NOT NULL;
     
     v_max number;
    
    BEGIN 
     OPEN c_suma;
     LOOP
     FETCH  c_suma INTO vc_suma;
        EXIT WHEN c_suma%NOTFOUND;
        UPDATE DWH_TRANSACCIONES_VISITAS SET
             TOTAL_TRAN = vc_suma where  COD_SEDE = inmo_sede;
     END LOOP;
     
    CLOSE c_suma;  
    
    update DWH_TRANSACCIONES_VISITAS SET COD_SEDE_MAX='FALSE';
    OPEN c_maximo;
    LOOP
    FETCH c_maximo INTO v_max;
       EXIT WHEN c_maximo%NOTFOUND;
       UPDATE DWH_TRANSACCIONES_VISITAS SET
              COD_SEDE_MAX ='TRUE' WHERE TOTAL_TRAN=v_max;
    END LOOP;
    
    CLOSE c_maximo;
    
   END; 
	 -- Consulta 5 
    IF inmo_empleado is not null then
        SELECT COUNT(anio) INTO n_any FROM DWH_TRANSACCIONES_VISITAS WHERE anio = inmo_anio AND COD_EMPLEADO=inmo_empleado;

      -- Si no existe el ao para ese empleado, se inserta
	     IF (n_any = 0) THEN
		      INSERT INTO DWH_TRANSACCIONES_VISITAS (ID,ANIO,COD_EMPLEADO,NUM_ACT_VISITAS,NUM_TRANSACCIONES,TOTAL_TRAN,TOTAL_VISITAS)
		      VALUES (estadistica_seq.nextval,inmo_anio,inmo_empleado,0,0,0,0);
	     END IF;

    -- Actualiza el nmero de VISITAS DEL EMPLEADO   
	     UPDATE DWH_TRANSACCIONES_VISITAS SET NUM_ACT_VISITAS = NUM_ACT_VISITAS + 1 WHERE (anio = inmo_anio) and (cod_empleado =inmo_empleado);
    end if;

  -- busqueda del empleado con mximo numero de visitas. El campo cod_empleado_max contendr si no est vacio el cdigo
   -- del empleado con mayor nmero de visitas.(Consulta 5)
  DECLARE 
      CURSOR c_suma IS 
      SELECT SUM(NUM_ACT_VISITAS)  FROM DWH_TRANSACCIONES_VISITAS WHERE ANIO < = EXTRACT(YEAR FROM SYSDATE)  AND COD_EMPLEADO = inmo_empleado;
        
       vc_suma NUMBER;    
    
    CURSOR c_maximo is
      SELECT MAX(TOTAL_VISITAS) FROM DWH_TRANSACCIONES_VISITAS WHERE COD_EMPLEADO IS NOT NULL;
     
     v_max number;
    
    BEGIN 
     OPEN c_suma;
     LOOP
     FETCH  c_suma INTO vc_suma;
        EXIT WHEN c_suma%NOTFOUND;
        UPDATE DWH_TRANSACCIONES_VISITAS SET
             TOTAL_VISITAS = vc_suma where  COD_EMPLEADO = inmo_empleado;
     END LOOP;
     
    CLOSE c_suma;  
    
    update DWH_TRANSACCIONES_VISITAS SET COD_EMPLEADO_MAX='FALSE';
    OPEN c_maximo;
    LOOP
    FETCH c_maximo INTO v_max;
       EXIT WHEN c_maximo%NOTFOUND;
       UPDATE DWH_TRANSACCIONES_VISITAS SET
              COD_EMPLEADO_MAX ='TRUE' WHERE TOTAL_VISITAS=v_max;
    END LOOP;
    
    CLOSE c_maximo;
    
   END; 
 
    INSERT INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva TRANSACCION ALQUILER  VENTA  REGIMEN VISITAS]','','OK',USER);

	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva TRANSACCION ALQUILER  VENTA  REGIMEN VISITAS]','',error_desc,USER);
	  COMMIT;

  END ALTA_DWH_TRANSACCIONES_VISITAS;

  --Consulta 2: Comercial que el momento de realizar la consulta tiene un porcentaje de exito mejor. Considerando el porcentaje de exito la ratio entre propuestas aceptadas y propuestas presentdas.

  /*Entendemos el momento de realizar la consulta como la fecha actual, se compararan todos los empleados de todos los aos desde el 
ao actual para atrs. Podra haber otras propuestas futuras dadas de alta en el sistema.

UNA PROPUESTA est aceptada ESTADO='ACEPTADA'. AL darla de alta el estado siempre es 'INICIADA' (esto rellenara el nmero de 
propuestas presentadas por el empleado en el momento) al darla de baja el estado es 'BAJA' por tanto slo puede ser 'ACEPTADA' en 
una modificacin (incrementaramos el nmero de propuestas presentadas por el empleado en el momento).

habr un campo mayor que guardar el cdigo del empleado que tenga myor nmero de propuestas ese ao. Para el resto de empleados 
estar vacio */  

  PROCEDURE ALTA_DWH_PROPUESTAS_ACEPTADAS (inmo_fecha DATE, inmo_empleado PROPUESTAS.CODIGO_EMPLEADO%TYPE) AS
  n_any VARCHAR2(4);
  error_desc   VARCHAR2(1000); -- descripcion del error
	e_error      EXCEPTION;      -- error controlado
  v_suma  NUMBER;

  BEGIN

   -- Verificamos si existe estadsticas de ao indicado para la Consulta2.  	
    IF (inmo_fecha is not null) and (inmo_empleado is not null) then
        SELECT COUNT(anio) INTO n_any FROM DWH_PROPUESTAS_ACEPTADAS WHERE anio = EXTRACT(YEAR FROM inmo_fecha);

      -- Si no existe el ao para ese empleado, se inserta
	     IF (n_any = 0) THEN
		      INSERT INTO DWH_PROPUESTAS_ACEPTADAS (ID,ANIO,FECHA_ALTA,COD_EMPLEADO,PROPUESTAS_ACEPTADAS,NUM_PROPUESTAS,TOTAL_ACEPTADAS,NUM_TOTAL_PROPUESTAS)
		      VALUES (estadistica_seq.nextval,EXTRACT(YEAR FROM inmo_fecha),null,inmo_empleado,0,0,0,0);
	     END IF;
        -- Actualiza el nmero de propuestas aceptadas del empleado  
	     UPDATE DWH_PROPUESTAS_ACEPTADAS SET PROPUESTAS_ACEPTADAS = PROPUESTAS_ACEPTADAS + 1 WHERE (anio = EXTRACT(YEAR FROM inmo_fecha)) and (cod_empleado =inmo_empleado);
     -- Actualizamos el campo TOTAL_ACEPTADAS con LA SUMA DE TODAS LAS propuestas ACEPTADAS DEL EMPLEADO en este momento en el sistema.
         select sum(propuestas_aceptadas)into v_suma FROM DWH_PROPUESTAS_ACEPTADAS group by cod_empleado
           having cod_empleado = inmo_empleado;  
       
         update dwh_propuestas_aceptadas set total_aceptadas = v_suma WHERE COD_EMPLEADO = inmo_empleado;
       
    
    ELSE
         error_desc:='ERROR: Parmetro FECHA Y CDIGO DE EMPLEADO obligatorio con valor nulo';
	       raise e_error;
   END IF;


    INSERT INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva PROPUESTA ACEPTADA DEL EMPLEADO:] '|| inmo_empleado ,'','OK',USER);

	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva PROPUESTA ACEPTADA DEL EMPLEADO:] '|| inmo_empleado ,'',error_desc,USER);
	  COMMIT;

  END ALTA_DWH_PROPUESTAS_ACEPTADAS;




-- PARTE DE LA Consulta2: Incrementa el nmero de propuesta de un empleado en el ao en curso y posteriormente realiza la suma de todas la propuestas que ha realizado
-- hasta el momento actualizando el valor del campo num_total_propuestas en el ao en curso.
PROCEDURE ALTA_DWH_NUM_PROPUESTAS (inmo_fecha IN DATE,inmo_empleado IN PROPUESTAS.CODIGO_EMPLEADO%TYPE) AS
 n_any VARCHAR2(4);
  error_desc   VARCHAR2(1000); -- descripcion del error
	e_error      EXCEPTION;      -- error controlado
  total_propuestas NUMBER:=0;
  v_suma NUMBER;

BEGIN
-- Verificamos si existe estadsticas de ao indicado para la Consulta2.  	
    IF (inmo_fecha is not null) and (inmo_empleado is not null) then
        SELECT COUNT(anio) INTO n_any FROM DWH_PROPUESTAS_ACEPTADAS WHERE anio = EXTRACT(YEAR FROM inmo_fecha)AND COD_EMPLEADO = inmo_empleado ;

      -- Si no existe el ao para ese empleado, se inserta
	     IF (n_any = 0) THEN
		      INSERT INTO DWH_PROPUESTAS_ACEPTADAS (ID,ANIO,FECHA_ALTA,COD_EMPLEADO,PROPUESTAS_ACEPTADAS,NUM_PROPUESTAS,TOTAL_ACEPTADAS,NUM_TOTAL_PROPUESTAS)
		      VALUES (estadistica_seq.nextval,EXTRACT(YEAR FROM inmo_fecha),null,inmo_empleado,0,0,0,0);
	     END IF;
        -- Actualiza el nmero de propuestas del empleado  
	     UPDATE DWH_PROPUESTAS_ACEPTADAS SET NUM_PROPUESTAS = NUM_PROPUESTAS + 1 WHERE (anio = EXTRACT(YEAR FROM inmo_fecha)) and (cod_empleado =inmo_empleado);

   
       -- Actualizamos el campo NUM_TOTAL_PROPUESTAS con LA SUMA DE TODAS LAS propuestas DEL EMPLEADO en este momento en el sistema.
         select sum(num_propuestas)into v_suma FROM DWH_PROPUESTAS_ACEPTADAS group by cod_empleado
           having cod_empleado = inmo_empleado;  
       
         update dwh_propuestas_aceptadas set num_total_propuestas = v_suma WHERE COD_EMPLEADO = inmo_empleado;
      
  
    ELSE
         error_desc:='ERROR: Parmetro FECHA Y CDIGO DE EMPLEADO obligatorio con valor nulo';
	       raise e_error;
   END IF;


    INSERT INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva PROPUESTA  DEL EMPLEADO:] '|| inmo_empleado ,'','OK',USER);

	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva PROPUESTA  DEL EMPLEADO:] '|| inmo_empleado ,'',error_desc,USER);
	  COMMIT;

END ALTA_DWH_NUM_PROPUESTAS;

-- Consulta3: Dado un ao concreto, importe total de las propuestas presentadas y que han sido aceptadas por alguno de nuestros clientes.
PROCEDURE ALTA_DWH_IMPORTE_PROPUESTAS (inmo_anio IN VARCHAR2,inmo_importe_total IN NUMBER) AS
  n_any VARCHAR2(4);
  error_desc   VARCHAR2(1000); -- descripcion del error
	e_error      EXCEPTION;      -- error controlado
  v_suma   NUMBER;


BEGIN
  IF (inmo_anio is not null) and (inmo_importe_total is not null) then
        SELECT COUNT(anio) INTO n_any FROM DWH_IMPORTE_ACEPTADAS WHERE anio = inmo_anio;
        

        IF (n_any) = 0 then 
            insert into dwh_importe_aceptadas values(inmo_anio,0);
	      end if;
      -- actauliza el importe total de las propuestas aceptadas para el ao introducido como parmetro.
       UPDATE DWH_IMPORTE_ACEPTADAS SET IMPORTE_TOTAL = IMPORTE_TOTAL + inmo_importe_total WHERE ANIO = inmo_anio;

   ELSE
         error_desc:='ERROR: Parmetro ao e importe total obligatorio  en ALTA_DWH_IMPORTE_PROPUESTAS con valor nulo';
	       raise e_error;
  END IF;
  INSERT INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: IMPORTE TOTAL PROPUESTAS ACEPTADAS:] '|| inmo_importe_total || 'para el ao: '||inmo_anio,'','OK',USER);

	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: IMPORTE TOTAL PROPUESTAS ACEPTADAS:] '|| inmo_importe_total || 'para el ao: '||inmo_anio ,'',error_desc,USER);
	  COMMIT; 


END ALTA_DWH_IMPORTE_PROPUESTAS;


 --Consulta 7: Por cada sede y ao porcentaje de inmuebles alquilados y vendidos
 -- Consulta9 : Pais que en el ao en curso lleva un numero mayor de transicciones (alquiler o venta). Si conocemos la sede, conocemos el pais.
 PROCEDURE ALTA_DWH_PORCENTAJE_TRANS (inmo_anio IN VARCHAR2, inmo_sede IN VARCHAR2) AS

  n_any VARCHAR2(4);
  error_desc   VARCHAR2(1000); -- descripcion del error
	e_error      EXCEPTION;      -- error controlado
  total_transacciones NUMBER:=0;
  v_pais DWH_PORCENTAJE_TRANSACCIONES.PAIS%TYPE;
  total_pais NUMBER:=0;
  v_max_tran NUMBER;
  v_max_pais NUMBER;
  v_max NUMBER;

 BEGIN
  -- Verificamos si existe estadsticas de ao indicado para la Consulta2.  	
    IF (inmo_anio is not null) and (inmo_sede is not null) then
      --- Consulta 7
		  SELECT COUNT(ANIO) into n_any from DWH_PORCENTAJE_TRANSACCIONES WHERE ANIO = inmo_anio AND COD_SEDE = inmo_sede;
      -- Localizamos el pais de la sede 
      SELECT pais into v_pais FROM SEDES where codigo_sede = inmo_sede;
      -- Si no existe el ao para esta sede se inserta
	     IF (TO_NUMBER(n_any) = 0) THEN
         INSERT INTO DWH_PORCENTAJE_TRANSACCIONES (ID,ANIO,PAIS,COD_SEDE,NUM_TRAN_SEDE_ANIO,NUM_TOTAL_TRAN,NUM_TOTAL_PAIS) VALUES
              (estadistica_seq.nextval,inmo_anio,v_pais,inmo_sede,0,0,0);
       END IF;
        -- Incrementa el nmero total de transaccione por sede y ao
         UPDATE DWH_PORCENTAJE_TRANSACCIONES SET NUM_TRAN_SEDE_ANIO = NUM_TRAN_SEDE_ANIO + 1 WHERE anio = inmo_anio AND cod_sede=inmo_sede;
       -- incrementa el total de transaciones por sede
         UPDATE DWH_PORCENTAJE_TRANSACCIONES SET NUM_TOTAL_TRAN = NUM_TOTAL_TRAN +1 WHERE cod_sede=inmo_sede;
         SELECT MAX(NUM_TOTAL_TRAN) into v_max_tran from dwh_porcentaje_transacciones WHERE cod_sede=inmo_sede;
         UPDATE DWH_PORCENTAJE_TRANSACCIONES SET NUM_TOTAL_TRAN = v_max_tran WHERE cod_sede=inmo_sede;
         
    -- Consulta 9 por pais
         UPDATE DWH_PORCENTAJE_TRANSACCIONES SET NUM_TOTAL_PAIS  =  NUM_TOTAL_PAIS +1 WHERE PAIS =v_pais;  
         SELECT MAX(NUM_TOTAL_PAIS)INTO v_max_pais from DWH_PORCENTAJE_TRANSACCIONES where PAIS= v_pais;
         UPDATE DWH_PORCENTAJE_TRANSACCIONES SET NUM_TOTAL_PAIS  = v_max_pais where PAIS= v_pais;
         UPDATE DWH_PORCENTAJE_TRANSACCIONES SET PAIS_MAYOR = 'FALSE';
         SELECT MAX(NUM_TOTAL_PAIS) INTO v_max FROM DWH_PORCENTAJE_TRANSACCIONES;
         UPDATE DWH_PORCENTAJE_TRANSACCIONES SET PAIS_MAYOR = 'TRUE' WHERE NUM_TOTAL_PAIS = v_max;
         
   ELSE
         error_desc:='ERROR: Parmetro AO  Y CDIGO DE LA SEDE obligatorios EN ALTA_DWH_PORCENTAJE_TRANSACCIONES';
	       raise e_error;
  END IF;

   COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: PORCENTAJE DE TRANSACCIONES POR SEDE:] '|| inmo_sede || 'para el ao: '||inmo_anio ,'',error_desc,USER);
	  COMMIT; 

 
END  ALTA_DWH_PORCENTAJE_TRANS;  

-- Consulta8 : Dado un ao cualquiera, porcentaje de inmuebles vendidos, sobre los que la empresa realiza tareas de gestin posterior a la venta.
-- Considero tareas de gestin posterior a la venta si existen actuaciones con proveedores.
 PROCEDURE ALTA_DWH_PORCENTAJE_POSVENTA (inmo_anio IN VARCHAR2,inmo_gestion IN NUMBER) AS
 
  n_any VARCHAR2(4);
  error_desc   VARCHAR2(1000); -- descripcion del error
	e_error      EXCEPTION;      -- error controlado

BEGIN
  -- Verificamos si existe estadsticas de ao indicado para la Consulta8.  	
    IF (inmo_anio is not null) THEN
        SELECT COUNT(anio) INTO n_any FROM DWH_PORCENTAJE_POSTERIOR_VENTA WHERE anio = inmo_anio;

        IF (n_any) = 0 then 
           -- genera el registro del ao correspondiente
           INSERT INTO DWH_PORCENTAJE_POSTERIOR_VENTA (ID,ANIO,NUM_VENDIDOS_POST,NUM_TOTAL) VALUES
                  (estadistica_seq.nextval,inmo_anio,0,0);
        END IF;   
        -- actauliza el importe total de INMUEBLES VENDIDOS para el ao
        UPDATE DWH_PORCENTAJE_POSTERIOR_VENTA SET NUM_TOTAL = NUM_TOTAL + 1  WHERE ANIO = inmo_anio;
        
        IF (inmo_gestion <> 0)  THEN
            UPDATE DWH_PORCENTAJE_POSTERIOR_VENTA SET NUM_VENDIDOS_POST = NUM_VENDIDOS_POST + 1 WHERE ANIO =inmo_anio;
        END IF;
        
     ELSE
         error_desc:='ERROR: Parmetro AO obligatorio EN ALTA_DWH_PORCENTAJE_POSVENTA';
	       raise e_error;
     
    END IF;

    COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: porcentaje postventa:] para el ao: '||inmo_anio ,'',error_desc,USER);
	  COMMIT; 
END ALTA_DWH_PORCENTAJE_POSVENTA;


--Consulta 10: Tipo de inmueble que ms se ha alquilado en cada uno de los pases en los que la empresa tiene sede. 
PROCEDURE ALTA_DWH_TIPOINMUEBLE_PAIS (inmo_inmueble IN VARCHAR2,inmo_pais IN VARCHAR2) AS

error_desc   VARCHAR2(1000); -- descripcion del error
e_error      EXCEPTION;      -- error controlado
v_descripcion VARCHAR2(200);
v_id NUMBER:=0;
BEGIN
  IF ( inmo_inmueble IS NOT NULL) AND ( inmo_pais IS NOT NULL) THEN
  -- Verificamos si existe estadsticas para el tipo de inmueble indicado y el pais.
  SELECT COUNT(ID) into v_id FROM DWH_TIPOINMUEBLE_MASALQUILADO   where TIPO_INMUEBLE = inmo_inmueble AND PAIS = inmo_pais;
  IF (v_id = 0) THEN
  -- generamos el registro
     INSERT INTO DWH_TIPOINMUEBLE_MASALQUILADO VALUES (estadistica_seq.nextval,inmo_pais,inmo_inmueble,0,'');
   END IF; 
  
  select descripcion into v_descripcion from tipo_inmueble  where codigo_tipo = inmo_inmueble ;
  UPDATE DWH_TIPOINMUEBLE_MASALQUILADO SET DESCRIPCION = v_descripcion where PAIS = inmo_pais AND TIPO_INMUEBLE = inmo_inmueble;
  UPDATE DWH_TIPOINMUEBLE_MASALQUILADO SET NUM_TOTAL_ALQUILERES = NUM_TOTAL_ALQUILERES + 1  WHERE PAIS = inmo_pais AND TIPO_INMUEBLE = inmo_inmueble;
    
  
  --- ACTUALIZAMOS LA TABLA AUXILIAR MAXIMOS PAISES INMUEBLES CONTIENE SOLO LOS VALORES MXIMOS PARA LA ESTADSTICA.
  execute immediate 'truncate table DWH_MAXIMOS_PAIS_INMUEBLE' ;
  DECLARE
      cursor c_maximo is
          select max(num_total_alquileres),pais from dwh_tipoinmueble_masalquilado group by pais;
      vc_maximo number;
      v_pais varchar2(20);
      v_descripcion varchar2(200);
  BEGIN
   OPEN c_maximo;
   LOOP
        FETCH  c_maximo INTO vc_maximo,v_pais;
        EXIT WHEN c_maximo%NOTFOUND;
         insert into DWH_MAXIMOS_PAIS_INMUEBLE values (vc_maximo,v_pais);
           
     END LOOP;
   close c_maximo;
  END;
  
  
  ELSE
         error_desc:='ERROR: Parmetro tipo de inmueble y pais obligatorios EN ALTA_DWH_TIPOINMUEBLE_PAIS';
	       raise e_error;
  
  END IF;

  

  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Tipo inmueble ms alquilado:] para el pais: '||inmo_pais ,'',error_desc,USER);
	  COMMIT; 

END ALTA_DWH_TIPOINMUEBLE_PAIS;

-- Consulta 11:  Nmero total de personas contratadas en cada uno de los pases en el momento de realizar la consulta. 
 PROCEDURE ALTA_DWH_CONTRATADOS_PAISES (inmo_pais IN VARCHAR2) AS
  error_desc   VARCHAR2(1000); -- descripcion del error
  e_error      EXCEPTION;      -- error controlado
  v_id NUMBER:=0;
 
 BEGIN
  IF ( inmo_pais IS NOT NULL) THEN
    -- Verificamos si existe  la estadstica porque al menos hay algn empleado contratado en el pais
    SELECT COUNT(ID) into v_id FROM DWH_CONTRATADOS_PAISES WHERE PAIS = inmo_pais;
    IF (v_id=0) THEN
       INSERT INTO DWH_CONTRATADOS_PAISES VALUES (estadistica_seq.nextval,inmo_pais,0);
    END IF;
    -- Actualizamos el contador de contratados en ese pais.
     UPDATE DWH_CONTRATADOS_PAISES SET NUM_TOTAL_CONTRATADOS = NUM_TOTAL_CONTRATADOS + 1 WHERE PAIS =inmo_pais;
 
  ELSE
         error_desc:='ERROR: Parmetro PAIS obligatorio en EN ALTA_DWH_CONTRATADOS_PAISES';
	       raise e_error;
  END IF;

    -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Personas contratadas:] para el pais: '||inmo_pais ,'',error_desc,USER);
	  COMMIT; 




 END ALTA_DWH_CONTRATADOS_PAISES;

-- Consulta 12: Entre todos los datos de que se disponga en la BD, cliente que ms litigios jurdicos haya presentado contra la empresa.
PROCEDURE ALTA_DWH_LITIGIOS_JURIDICOS (inmo_cliente IN VARCHAR2) AS

error_desc   VARCHAR2(1000); -- descripcion del error
e_error      EXCEPTION;      -- error controlado
v_id NUMBER:=0;
v_cliente DWH_LITIGIOS_JURIDICOS.COD_CLIENTE%TYPE;


BEGIN
IF ( inmo_cliente IS NOT NULL) THEN
    -- Verificamos si existe  la estadstica que el cliente ya ha presentado litigio jurdico contra la empresa.
    SELECT COUNT(ID) into v_id FROM DWH_LITIGIOS_JURIDICOS WHERE COD_CLIENTE = inmo_cliente;
    IF (v_id=0) THEN
       INSERT INTO DWH_LITIGIOS_JURIDICOS VALUES (estadistica_seq.nextval,inmo_cliente,0,'FALSE');
    END IF;
    -- Actualizamos el contador de litigios de ese cliente.
     UPDATE DWH_LITIGIOS_JURIDICOS SET NUM_TOTAL_LITIGIOS = NUM_TOTAL_LITIGIOS + 1 WHERE COD_CLIENTE =inmo_cliente;
 
 --- Localizamos el cliente con mayor nmero de litigios y actualizamos el campo ES_MAYOR ( Si es el mayor tendr valor TRUE ) , se puede dar el caso de ms de uno.
  DECLARE 
   CURSOR c_cliente IS 
     SELECT COD_CLIENTE FROM DWH_LITIGIOS_JURIDICOS WHERE NUM_TOTAL_LITIGIOS = (SELECT MAX(NUM_TOTAL_LITIGIOS) FROM DWH_LITIGIOS_JURIDICOS);
   vc_cliente  VARCHAR2(20);
  BEGIN
   --- ACTUALIZAMOS VALORES
    UPDATE DWH_LITIGIOS_JURIDICOS SET ES_MAYOR='FALSE';
   -- Actualizamos a true los clientes con mayor nmero de litigios
   OPEN c_cliente;
   LOOP
    FETCH  c_cliente INTO vc_cliente;
        EXIT WHEN c_cliente%NOTFOUND;
     UPDATE DWH_LITIGIOS_JURIDICOS SET ES_MAYOR='TRUE' WHERE COD_CLIENTE = vc_cliente;
   END LOOP;
  CLOSE c_cliente;
  
  END;
 
 
  ELSE
         error_desc:='ERROR: Parmetro CLIENTE obligatorio en EN ALTA_DWH_LITIGIOS_JURIDICOS';
	       raise e_error;
  END IF;

   -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Litigios Jurdicos:] para el cliente: '||inmo_cliente ,'',error_desc,USER);
	  COMMIT; 

END ALTA_DWH_LITIGIOS_JURIDICOS;


-- Consulta 13 : En el ao en curso, cliente al que ms ha facturado la empresa por cualquier concepto
PROCEDURE ALTA_DWH_FACTURACION_CLIENTE (inmo_cliente IN VARCHAR2, inmo_facturacion IN NUMBER) AS
error_desc   VARCHAR2(1000); -- descripcion del error
e_error      EXCEPTION;      -- error controlado
v_id NUMBER:=0;
v_cliente DWH_FACTURACION_CLIENTE.COD_CLIENTE%TYPE;

BEGIN
IF ( inmo_cliente IS NOT NULL) THEN
    -- Verificamos si existe  la estadstica que la empresa ha facturado al cliente en el ao en curso.
    SELECT COUNT(ID) into v_id FROM DWH_FACTURACION_CLIENTE WHERE COD_CLIENTE = inmo_cliente AND ANIO = EXTRACT (YEAR FROM SYSDATE);
    IF (v_id=0) THEN
       INSERT INTO DWH_FACTURACION_CLIENTE VALUES (estadistica_seq.nextval,EXTRACT (YEAR FROM SYSDATE),inmo_cliente,0,'FALSE');
    END IF;
    -- Actualizamos el contador de facturacion del cliente en el ao en curso
     UPDATE DWH_FACTURACION_CLIENTE SET IMPORTE_TOTAL = IMPORTE_TOTAL + inmo_facturacion WHERE COD_CLIENTE =inmo_cliente AND ANIO = EXTRACT (YEAR FROM SYSDATE);
 
 --- Localizamos el cliente al que ms se ha facturado y actualizamos el campo ES_MAYOR ( Si es el mayor tendr valor TRUE ) , se puede dar el caso de ms de uno.
  DECLARE 
   CURSOR c_cliente IS 
     SELECT COD_CLIENTE FROM DWH_FACTURACION_CLIENTE  WHERE  IMPORTE_TOTAL = (SELECT MAX(IMPORTE_TOTAL) FROM DWH_FACTURACION_CLIENTE WHERE ANIO=EXTRACT (YEAR FROM SYSDATE) );
   vc_cliente  VARCHAR2(20);
  BEGIN
   --- ACTUALIZAMOS VALORES
    UPDATE DWH_FACTURACION_CLIENTE SET MAYOR_FAC='FALSE' WHERE ANIO= EXTRACT (YEAR FROM SYSDATE);
   -- Actualizamos a true los clientes con mayor nmero de litigios
   OPEN c_cliente;
   LOOP
    FETCH  c_cliente INTO vc_cliente;
        EXIT WHEN c_cliente%NOTFOUND;
     UPDATE DWH_FACTURACION_CLIENTE SET MAYOR_FAC='TRUE' WHERE COD_CLIENTE = vc_cliente AND ANIO = EXTRACT (YEAR FROM SYSDATE);
   END LOOP;
  CLOSE c_cliente;
  
  END;
 
 
  ELSE
         error_desc:='ERROR: Parmetro CLIENTE obligatorio en EN ALTA_DWH_FACTURACION_CLIENTE';
	       raise e_error;
  END IF;

   -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: facturacin:]  al cliente: '||inmo_cliente ,'',error_desc,USER);
	  COMMIT; 


END ALTA_DWH_FACTURACION_CLIENTE;

-- Consulta 14:  En una sede concreta, nmero de inmuebles disponibles en el momento de realizar la consulta.
PROCEDURE ALTA_DWH_INMUEBLES_DISPONIBLES (inmo_sede IN VARCHAR2) AS
error_desc   VARCHAR2(1000); -- descripcion del error
e_error      EXCEPTION;      -- error controlado
v_id NUMBER:=0;

BEGIN
IF ( inmo_sede IS NOT NULL) THEN
    -- Verificamos si existe  la estadstica del nmero de inmuebles disponibles en la sede.
    SELECT COUNT(ID) into v_id FROM DWH_INMUEBLES_DISPONIBLES WHERE COD_SEDE = inmo_sede;
    IF (v_id=0) THEN
       INSERT INTO DWH_INMUEBLES_DISPONIBLES VALUES (estadistica_seq.nextval,inmo_sede,0);
    END IF;
    -- Actualizamos el nmero de inmuebles disponibles para la sede.
     UPDATE DWH_INMUEBLES_DISPONIBLES SET NUM_TOTAL_INMUEBLES = NUM_TOTAL_INMUEBLES + 1 WHERE COD_SEDE =inmo_sede;
 ELSE
 
         error_desc:='ERROR: Parmetro SEDE obligatorio en EN ALTA_DWH_INMUEBLES_DISPONIBLES';
	       raise e_error;
 END IF;

 -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Inmuebles disponibles :]  para la sede '||inmo_sede ,'',error_desc,USER);
	  COMMIT; 



END ALTA_DWH_INMUEBLES_DISPONIBLES;

PROCEDURE ALTA_DWH_PORCENTAJE_PROVE (inmo_anio IN VARCHAR2,inmo_mes IN VARCHAR2) AS
error_desc   VARCHAR2(1000); -- descripcion del error
e_error      EXCEPTION;      -- error controlado
v_id    number;
v_anio_anterior varchar2(4);
v_anio_siguiente varchar2(4);

BEGIN
IF ( inmo_anio IS NOT NULL) and (inmo_mes IS NOT NULL) THEN
    -- Resolvemos anio anterior y siguiente
    v_anio_anterior := to_char(to_number(inmo_anio)-1);
    v_anio_siguiente := to_char(to_number(inmo_anio)+1);
    -- Verificamos si existe  la estadstica del nmero de inmuebles disponibles en la sede.
    SELECT COUNT(ID) into v_id FROM DWH_PORCENTAJE_PROVEEDORES WHERE ANIO = inmo_anio AND MES= inmo_mes AND ANIO_ANTERIOR =v_anio_anterior;
    IF (v_id=0) THEN
       INSERT INTO DWH_PORCENTAJE_PROVEEDORES VALUES (estadistica_seq.nextval,inmo_anio,inmo_mes,0,v_anio_anterior,0);
    END IF;
    -- Actualizamos el nmero de actuaciones
     UPDATE DWH_PORCENTAJE_PROVEEDORES SET NUM_ACTUACIONES = NUM_ACTUACIONES + 1 WHERE ANIO=inmo_anio AND MES=inmo_mes AND ANIO_ANTERIOR = v_anio_anterior;

  --- Actualizamos las actuaciones anterior
  select count(id) into v_id from DWH_PORCENTAJE_PROVEEDORES where anio = v_anio_siguiente and anio_anterior = inmo_anio and mes = inmo_mes;
  IF (v_id = 0) THEN
     INSERT INTO DWH_PORCENTAJE_PROVEEDORES VALUES (estadistica_seq.nextval,v_anio_siguiente,inmo_mes,0,inmo_anio,0);
  end if; 
   UPDATE DWH_PORCENTAJE_PROVEEDORES SET NUM_ACTUACIONES_ANTERIOR = NUM_ACTUACIONES_ANTERIOR +1 WHERE ANIO= v_anio_siguiente AND MES= inmo_mes AND ANIO_ANTERIOR = inmo_anio;
   
     
 ELSE
 
         error_desc:='ERROR: Parmetro AO Y MES  obligatorio en EN ALTA_DWH_PORCENTAJE_PROVE';
	       raise e_error;
 END IF;

 -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INMO_LOGS(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: porcentaje actuaciones proveedores :]  para el ao '||inmo_anio||'y el mes'||inmo_mes ,'',error_desc,USER);
	  COMMIT; 

END ALTA_DWH_PORCENTAJE_PROVE;
END INMO_MOD_ESTADISTICAS;