create or replace PACKAGE BODY INMO_MOD_GENERAL AS

-- Alta de Clientes

  PROCEDURE INMO_ALTA_CLIENTES (
                                inmo_nombre IN CLIENTES.nombre%TYPE,
				                        inmo_direccion IN CLIENTES.direccion%TYPE,
				                        inmo_cif IN CLIENTES.CIF%TYPE,
                                inmo_tef IN CLIENTES.TELEFONO%TYPE,
                                inmo_email IN CLIENTES.EMAIL%TYPE,
                                RSP OUT varchar2,
                                inmo_cliente OUT CLIENTES.codigo_cliente%TYPE
				
) AS

  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  COD_NUEVO_CLIENTE VARCHAR2(20);
  v_estado VARCHAR2(6):='ALTA';
  BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Nombre: '||inmo_nombre||', Direccion: '||inmo_direccion||', CIF: '||inmo_cif||', Telefono: '|| inmo_tef||', Email: '||inmo_email||', Estado: '||v_estado;

 --Verifica que todos los campos obligatorios tienen valor
	if (inmo_nombre is NULL or inmo_direccion is NULL or inmo_cif is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

  

 -- Aadimos nuevo cliente
 -- EL cdigo de cliente ser un varchar con formato YYYY/SECUENCIA, DONDE yyyy es el ao en curso


  INSERT INTO CLIENTES(codigo_cliente,nombre,direccion,cif,telefono,email,estado) 
     VALUES ('CLI/000'||TO_CHAR(cliente_seq.nextval),inmo_nombre,inmo_direccion,inmo_cif,inmo_tef,inmo_email,v_estado);
  RSP := 'OK - CLI/000'||TO_CHAR(cliente_seq.currval) ;
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta cliente]INMO_MOD_GENERAL.INMO_ALTA_CLIENTES', parametrosIN, RSP,user);


  COMMIT;





  -- Tratamiento de excepciones

EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Cliente]INMO_MOD_GENERAL.INMO_ALTA_CLIENTES', parametrosIN, error_desc,user);
      COMMIT;

  END INMO_ALTA_CLIENTES;

-- Baja de un cliente
  PROCEDURE INMO_BAJA_CLIENTES (inmo_codcliente IN CLIENTES.CODIGO_CLIENTE%TYPE, RSP OUT VARCHAR2) AS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_cliente number:=0;
  v_BAJA CONSTANT VARCHAR2(6) := 'BAJA'; 
  BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Cliente: '||inmo_codcliente;

   -- Verificamos que el cliente existe
   BEGIN
    SELECT 	count(codigo_cliente)
    INTO	num_cliente
    FROM	CLIENTES
    WHERE CODIGO_CLIENTE = inmo_codcliente; 
  END;
  IF (num_cliente = 0) THEN error_desc:='ERROR: El Cliente indicado no existe'; RAISE e_error; END IF;


  -- Los clientes no se eliminan. Se modifica su estado a BAJA
  UPDATE CLIENTES SET ESTADO = v_BAJA WHERE CODIGO_CLIENTE = inmo_codcliente;

 	RSP := 'OK';

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja del Cliente]INMO_MOD_GENERAL.INMO_BAJA_CLIENTES', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja del Cliente]INMO_MOD_GENERAL.INMO_BAJA_CLIENTES', parametrosIN, error_desc,user);
  COMMIT;  

  END INMO_BAJA_CLIENTES;

  -- Modificacin de un cliente, el estado slo se modifica mediante el proceso de Baja

  PROCEDURE INMO_MOD_CLIENTES ( inmo_cod_cliente IN CLIENTES.codigo_cliente%TYPE,
						                    inmo_nombre IN CLIENTES.nombre%TYPE,
				                        inmo_direccion IN CLIENTES.direccion%TYPE,
				                        inmo_cif IN CLIENTES.CIF%TYPE,
                                inmo_tef IN CLIENTES.TELEFONO%TYPE,
                                inmo_email IN CLIENTES.EMAIL%TYPE,	
						RSP OUT varchar2) AS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);         

  BEGIN
  	-- Concatenamos los parametros de entrada para el LOG, el estado slo se modifica en el proceso de Baja.
	parametrosIN := 'Cdigo cliente: '||inmo_cod_cliente||', Nombre: '||inmo_nombre||', Direccion: '||inmo_direccion||', CIF: '||inmo_cif||', Telfono: '||inmo_tef||', Email: '||inmo_email;

	 --Verifica que el parmetro Cdigo de Cliente no es nulo
	if (inmo_cod_cliente is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;


	-- Actualiza valores del cliente indicado
	UPDATE CLIENTES
	SET	NOMBRE = NVL(inmo_nombre,NOMBRE),
		  DIRECCION = NVL(inmo_direccion,DIRECCION),
		  CIF = NVL(inmo_cif,CIF),
		  TELEFONO = NVL(inmo_tef,TELEFONO),
		  EMAIL = NVL(inmo_email,EMAIL)
	WHERE CODIGO_CLIENTE = inmo_cod_cliente;

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin del Cliente]INMO_MOD_GENERAL.INMO_MOD_CLIENTES', parametrosIN, 'OK',user);	
	RSP:='OK';
	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin del Cliente]INMO_MOD_GENERAL.INMO_MOD_CLIENTES', parametrosIN, error_desc,user);
	  COMMIT;  


  END INMO_MOD_CLIENTES;


  -- alta de la sede
  PROCEDURE INMO_ALTA_SEDES (
                                inmo_pais IN SEDES.pais%TYPE,
				                        inmo_zona IN SEDES.zona%TYPE,
				                        inmo_provincia IN SEDES.provincia%TYPE,
                                inmo_poblacion IN SEDES.poblacion%TYPE,
                                inmo_direccion IN SEDES.direccion%TYPE, 
                                inmo_fecha_ina IN SEDES.fecha_inauguracion%TYPE,
                                inmo_responsable IN SEDES.responsable%TYPE,
                                RSP OUT varchar2,
                                inmo_sede OUT SEDES.CODIGO_SEDE%TYPE

) AS

 parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  v_estado VARCHAR2(6):='ALTA';


BEGIN

 -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Pais: '||inmo_pais||', Zona: '||inmo_zona||', Provincia: '||inmo_provincia||', Poblacion: '|| inmo_poblacion||', Direccin: '||inmo_direccion||' Fecha Inauguracin: '||inmo_fecha_ina||', Responsable: '||inmo_responsable||', Estado: '||v_estado;

 --Verifica que todos los campos obligatorios tienen valor
	if (inmo_pais is NULL or inmo_poblacion is NULL or inmo_direccion is NULL or inmo_responsable IS NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;


 -- Aadimos nueva sede
   INSERT INTO SEDES (codigo_sede,pais,zona,provincia,poblacion,direccion,fecha_inauguracion,responsable,estado)
     VALUES ('SED/000'||TO_CHAR(sedes_seq.nextval),inmo_pais,inmo_zona,inmo_provincia,inmo_poblacion,inmo_direccion,inmo_fecha_ina,inmo_responsable,v_estado);
  RSP := 'OK SED/000'||TO_CHAR(sedes_seq.currval);
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta sede]INMO_MOD_GENERAL.INMO_ALTA_SEDES', parametrosIN, RSP,user);


  COMMIT;


EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Sede]INMO_MOD_GENERAL.INMO_ALTA_SEDES', parametrosIN, error_desc,user);
      COMMIT;

END INMO_ALTA_SEDES;

-- BAJA DE UNA SEDE
PROCEDURE INMO_BAJA_SEDES (inmo_cod_sede IN SEDES.CODIGO_SEDE%TYPE, RSP OUT VARCHAR2) AS


  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_sede number:=0;
  v_BAJA CONSTANT VARCHAR2(6) := 'BAJA'; 

  BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Sede: '||inmo_cod_sede;

   -- Verificamos que la sede existe
   BEGIN
    SELECT 	count(codigo_sede)
    INTO	num_sede
    FROM	SEDES
    WHERE CODIGO_SEDE = inmo_cod_sede; 
  END;
  IF (num_sede = 0) THEN error_desc:='ERROR: La sede indicada no existe'; RAISE e_error; END IF;


  --  Se modifica su estado a BAJA, no se elimina
  UPDATE SEDES SET ESTADO = v_BAJA WHERE CODIGO_SEDE = inmo_cod_sede;

 	RSP := 'OK' ;

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja de la Sede]INMO_MOD_GENERAL.INMO_BAJA_SEDES', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja de la Sede]INMO_MOD_GENERAL.INMO_BAJA_SEDES', parametrosIN, error_desc,user);
  COMMIT;  

END INMO_BAJA_SEDES;

 -- Modificacin de un cliente, el estado slo se modifica mediante el proceso de Baja

  PROCEDURE INMO_MOD_SEDES ( inmo_cod_sede IN SEDES.codigo_sede%TYPE,
						                   inmo_pais IN SEDES.pais%TYPE,
				                        inmo_zona IN SEDES.zona%TYPE,
				                        inmo_provincia IN SEDES.provincia%TYPE,
                                inmo_poblacion IN SEDES.poblacion%TYPE,
                                inmo_direccion IN SEDES.direccion%TYPE, 
                                inmo_fecha_ina IN SEDES.fecha_inauguracion%TYPE,
                                inmo_responsable IN SEDES.responsable%TYPE,
						RSP OUT varchar2) AS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);  
  v_estado varchar2(6):='ALTA';

  BEGIN
  	-- Concatenamos los parametros de entrada para el LOG, el estado slo se modifica en el proceso de Baja.
	 parametrosIN := 'Pais: '||inmo_pais||', Zona: '||inmo_zona||', Provincia: '||inmo_provincia||', Poblacion: '|| inmo_poblacion||', Direccin: '||inmo_direccion||' Fecha Inauguracin: '||inmo_fecha_ina||', Responsable: '||inmo_responsable||', Estado:: '||v_estado;

	 --Verifica que el parmetro Cdigo de Cliente no es nulo
	if (inmo_cod_sede is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;


	-- Actualiza valores del cliente indicado
	UPDATE SEDES
	SET	PAIS = NVL(inmo_pais,PAIS),
      ZONA = NVL(inmo_zona,ZONA),
      PROVINCIA = NVL(inmo_provincia,PROVINCIA),
      POBLACION = NVL(inmo_poblacion,Poblacion),
		  DIRECCION = NVL(inmo_direccion,DIRECCION),
		  FECHA_INAUGURACION = NVL(inmo_fecha_ina,FECHA_INAUGURACION),
		  RESPONSABLE = NVL(inmo_responsable,RESPONSABLE)

	WHERE CODIGO_SEDE = inmo_cod_sede;

   RSP:='OK. Modificada la sede con codigo: '|| inmo_cod_sede;	

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la Sede]INMO_MOD_GENERAL.INMO_MOD_SEDES', parametrosIN, RSP,user);	

	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la sede]INMO_MOD_GENERAL.INMO_MOD_SEDES', parametrosIN, error_desc,user);
	  COMMIT;  


  END INMO_MOD_SEDES;


-- Alta de un nuevo inmueble
PROCEDURE INMO_ALTA_INMUEBLE (
                                inmo_pais IN INMUEBLES.pais%TYPE,
				                        inmo_zona IN INMUEBLES.zona%TYPE,
				                        inmo_provincia IN INMUEBLES.provincia%TYPE,
                                inmo_poblacion IN INMUEBLES.poblacion%TYPE,
                                inmo_cp IN INMUEBLES.CODIGO_POSTAL%TYPE,
                                inmo_direccion IN INMUEBLES.direccion%TYPE, 
                                inmo_coor IN INMUEBLES.coordenadas_map%TYPE,
                                inmo_fecha_cons IN INMUEBLES.fecha_construccion%TYPE,
                                inmo_propietario IN INMUEBLES.propietario%TYPE,
                                inmo_tipo IN INMUEBLES.tipo_inmueble%TYPE,
                                inmo_carac IN INMUEBLES.caracteristicas%TYPE,
                                inmo_empleado IN EMPLEADOS.CODIGO_EMPLEADO%TYPE,
				                        RSP OUT varchar2,
                                inmo_cod_inmueble OUT INMUEBLES.CODIGO_INMUEBLE%TYPE

)AS
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  COD_NUEVO_INMUEBLE VARCHAR2(20);  
  v_estado VARCHAR2(6):='ALTA';
  num_inmueble number:=0;
  C_codigo_act_alta varchar2(20):='TAIALTA';
  C_des_empleado varchar2(100);
  C_situacion varchar2(100):='INICIADA';


BEGIN

 -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Pais: '||inmo_pais||', Zona: '||inmo_zona||', Provincia: '||inmo_provincia||', Poblacion: '||inmo_poblacion||', Cdigo Postal: '|| inmo_cp||', Direccin: '||inmo_direccion||', Cdigo Postal: '||inmo_cp||' Fecha construccin: '||inmo_fecha_cons||', Propietario: '||inmo_propietario||', Tipo: '||inmo_tipo||', Caractersticas: '||inmo_carac||', Estado: '||v_estado;

 --Verifica que todos los campos obligatorios tienen valor
	if (inmo_pais is NULL or inmo_zona is Null or inmo_poblacion is NULL or inmo_direccion is NULL or inmo_propietario IS NULL or inmo_tipo IS NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;
  -- Verifica que existe el tipo de inmueble
  BEGIN
    SELECT 	count(codigo_tipo)
    INTO	num_inmueble
    FROM	TIPO_INMUEBLE
    WHERE CODIGO_TIPO = inmo_tipo; 
  END;
  IF (num_inmueble = 0) THEN error_desc:='ERROR: EL cdigo del tipo de inmueble indicado no existe'; RAISE e_error; END IF;


 -- Aadimos nuevo inmueble
   INSERT INTO INMUEBLES (codigo_inmueble,pais,zona,provincia,poblacion,codigo_postal,direccion,coordenadas_map,fecha_construccion,fecha_entrada,propietario,tipo_inmueble,caracteristicas,estado)
     VALUES ('INM/000'||TO_CHAR(inmuebles_seq.nextval),inmo_pais,inmo_zona,inmo_provincia,inmo_poblacion,inmo_cp,inmo_direccion,inmo_coor,inmo_fecha_cons,SYSDATE,inmo_propietario,inmo_tipo,inmo_carac,v_estado);

  -- Verificamos que el empleado que da de alta la actuacin ocupa el puesto de comercial.
  
  SELECT DESCRIPCION INTO C_des_empleado FROM PUESTOS_TRABAJO WHERE  CODIGO_PUESTO = (SELECT CODIGO_PUESTO_TRABAJO FROM EMPLEADOS_SEDES WHERE (CODIGO_EMPLEADO = inmo_empleado) AND (FECHA_BAJA IS NULL));
  -- Generamos el registro Actuacin Inmuenbles (detalle)
  IF C_des_empleado ='COMERCIAL' THEN
     INSERT INTO ACTUACIONES_INMUEBLES (codigo_actuacion,codigo_tipo_actuacion,codigo_propietario,codigo_empleado,fecha_alta,situacion,cod_inmueble)
              VALUES (TO_CHAR(actuacion_seq.nextval), C_codigo_act_alta,inmo_propietario, inmo_empleado,SYSDATE, C_situacion,'INM/000'||TO_CHAR(inmuebles_seq.currval));

  ELSE
    error_desc:='ERROR: Un empleado que no trabaje con puesto de COMERCIAL, no puede dar de alta un INMUEBLE en el sistema'; RAISE e_error;
  END IF;

  RSP := 'OK';
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta inmueble]INMO_MOD_GENERAL.INMO_ALTA_INMUEBLE', parametrosIN, RSP,user);

  COMMIT;

  -- Tratamiento de excepciones

EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Inmueble]INMO_MOD_GENERAL.INMO_ALTA_INMUEBLE', parametrosIN, error_desc,user);
      COMMIT;



END INMO_ALTA_INMUEBLE;

--Baja de un inmueble
PROCEDURE INMO_BAJA_INMUEBLES (inmo_cod_inmueble IN INMUEBLES.CODIGO_INMUEBLE%TYPE, RSP OUT VARCHAR2) AS

  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_inmueble number:=0;
  v_BAJA CONSTANT VARCHAR2(6) := 'BAJA'; 

  BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Inmueble: '||inmo_cod_inmueble;

   -- Verificamos que el inmueble existe
   BEGIN
    SELECT 	count(codigo_inmueble)
    INTO	num_inmueble
    FROM	INMUEBLES
    WHERE CODIGO_INMUEBLE = inmo_cod_inmueble;
  END;
  IF (num_inmueble = 0) THEN error_desc:='ERROR: El inmueble indicado no existe'; RAISE e_error; END IF;


  --  Se modifica su estado a BAJA, no se elimina
  UPDATE INMUEBLES SET ESTADO = v_BAJA WHERE CODIGO_INMUEBLE = inmo_cod_inmueble;

 	RSP := 'OK' ;

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja de Inmueble]INMO_MOD_GENERAL.INMO_BAJA_INMUEBLES', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja de Inmuebles]INMO_MOD_GENERAL.INMO_BAJA_INMUEBLES', parametrosIN, error_desc,user);
  COMMIT;  

END INMO_BAJA_INMUEBLES;

--Modificacin de un inmueble
PROCEDURE INMO_MOD_INMUEBLES (  
                                inmo_cod_inmueble IN INMUEBLES.codigo_inmueble%TYPE,
                                inmo_pais IN INMUEBLES.pais%TYPE,
				                        inmo_zona IN INMUEBLES.zona%TYPE,
				                        inmo_provincia IN INMUEBLES.provincia%TYPE,
                                inmo_poblacion IN INMUEBLES.poblacion%TYPE,
                                inmo_cp IN INMUEBLES.CODIGO_POSTAL%TYPE,
                                inmo_direccion IN INMUEBLES.direccion%TYPE,
                                inmo_coor IN INMUEBLES.coordenadas_map%TYPE,
                                inmo_fecha_cons IN INMUEBLES.fecha_construccion%TYPE,
                                inmo_propietario IN INMUEBLES.propietario%TYPE,
                                inmo_tipo IN INMUEBLES.tipo_inmueble%TYPE,
                                inmo_carac IN INMUEBLES.caracteristicas%TYPE,
                                inmo_estado IN INMUEBLES.ESTADO%TYPE,
				                        RSP OUT varchar2
						)AS


  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);  
  v_estado varchar2(6):='ALTA';

  BEGIN
  	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Pais: '||inmo_pais||', Zona: '||inmo_zona||', Provincia: '||inmo_provincia||', Poblacion: '||inmo_poblacion||', Cdigo Postal: '|| inmo_cp||', Direccin: '||inmo_direccion||', Cdigo Postal: '||inmo_cp||' Fecha construccin: '||inmo_fecha_cons||', Propietario: '||inmo_propietario||', Tipo: '||inmo_tipo||', Caractersticas: '||inmo_carac||', Estado: '||v_estado;


  	 --Verifica que el parmetro Cdigo del inmueble no es nulo
	if (inmo_cod_inmueble is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;


	-- Actualiza valores del cliente indicado
	UPDATE INMUEBLES
	SET	PAIS = NVL(inmo_pais,PAIS),
      ZONA = NVL(inmo_zona,ZONA),
      PROVINCIA = NVL(inmo_provincia,PROVINCIA),
      POBLACION = NVL(inmo_poblacion,Poblacion),
      CODIGO_POSTAL = NVL(inmo_cp,CODIGO_POSTAL),
		  DIRECCION = NVL(inmo_direccion,DIRECCION),
      COORDENADAS_MAP = NVL(inmo_coor,COORDENADAS_MAP),
		  FECHA_CONSTRUCCION = NVL(inmo_fecha_cons,FECHA_CONSTRUCCION),
		  PROPIETARIO = NVL(inmo_propietario,PROPIETARIO),
      TIPO_INMUEBLE = NVL(inmo_tipo,TIPO_INMUEBLE),
      CARACTERISTICAS = NVL(inmo_carac,CARACTERISTICAS)	 
	WHERE CODIGO_INMUEBLE = inmo_cod_inmueble;

   RSP:='OK. Modificada el inmueble con codigo: '|| inmo_cod_inmueble;	

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin del INMUEBLE]INMO_MOD_GENERAL.INMO_MOD_INMUEBLES', parametrosIN, RSP,user);	

	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin del inmueble]INMO_MOD_GENERAL.INMO_MOD_INMUEBLES', parametrosIN, error_desc,user);
	  COMMIT;  



  END INMO_MOD_INMUEBLES;


-- Alta Imagenes
PROCEDURE INMO_ALTA_IMAGENES (
        inmo_nombre IN IMAGENES.nombre%TYPE,
				inmo_ruta IN IMAGENES.ruta%TYPE,
				inmo_inmueble IN IMAGENES.CODIGO_INMUEBLE%TYPE,
        inmo_imagen OUT IMAGENES.codigo_imagen%TYPE,
				 RSP OUT varchar2
) AS
num_inmueble VARCHAR2(20);
error_desc varchar2(1000); -- descripcion del error
 e_error exception; --error controlado
 parametrosIN varchar2(1000);  
 COD_NUEVA_IMAGEN varchar2(20);

BEGIN
-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Fichero: '||inmo_nombre||', Ruta: '||inmo_ruta||', Inmueble: '||inmo_inmueble;

 --Verifica que todos los campos obligatorios tienen valor
	if (inmo_nombre is NULL or inmo_ruta is Null or inmo_inmueble is NULL ) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;
  -- Verifica que existe el inmueble al que corresponde la imagen
  BEGIN
    SELECT 	count(codigo_inmueble)
    INTO	num_inmueble
    FROM	INMUEBLES
    WHERE CODIGO_INMUEBLE = inmo_inmueble; 
  END;
  IF (num_inmueble = 0) THEN error_desc:='ERROR: EL cdigo del inmueble indicado no existe'; RAISE e_error; END IF;


 -- Aadimos nuevo inmueble
   INSERT INTO IMAGENES (codigo_imagen,nombre,ruta,codigo_inmueble)
     VALUES (TO_CHAR(imagenes_seq.nextval),inmo_nombre,inmo_ruta,inmo_inmueble);


  RSP := 'OK';
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta imagenes]INMO_MOD_GENERAL.INMO_ALTA_IMAGENES', parametrosIN, RSP,user);

  COMMIT;



  -- Tratamiento de excepciones

EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Imagenes]INMO_MOD_GENERAL.INMO_ALTA_IMAGENES', parametrosIN, error_desc,user);
      COMMIT;




END INMO_ALTA_IMAGENES;

--Baja de una imagen
PROCEDURE INMO_BAJA_IMAGENES (inmo_imagen IN IMAGENES.CODIGO_IMAGEN%TYPE, RSP OUT VARCHAR2) AS

 error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_imagen number:=0;

BEGIN
-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Imagen: '||inmo_imagen;

   -- Verificamos que el inmueble existe
   BEGIN
    SELECT 	count(codigo_imagen)
    INTO	num_imagen
    FROM	IMAGENES
    WHERE CODIGO_IMAGEN = inmo_imagen;
  END;
  IF (num_imagen = 0) THEN error_desc:='ERROR: La imagen indicada no existe'; RAISE e_error; END IF;

  --- Eliminamos la imagen en este caso, ya que ninguna otra tabla quedar obsoleta.
  DELETE FROM IMAGENES WHERE CODIGO_IMAGEN= inmo_imagen;
  RSP := 'OK';

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja de la Imagen]INMO_MOD_GENERAL.INMO_BAJA_IMAGENES', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja de la Imagen]INMO_MOD_GENERAL.INMO_BAJA_IMAGENES', parametrosIN, error_desc,user);
  COMMIT;  

END INMO_BAJA_IMAGENES;

-- Modificacin de una imagen
PROCEDURE INMO_MOD_IMAGENES ( inmo_imagen IMAGENES.CODIGO_IMAGEN%TYPE,
                        inmo_nombre IN IMAGENES.nombre%TYPE,
				                inmo_ruta IN IMAGENES.RUTA%TYPE,
				                RSP OUT varchar2 ) as


  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);  
  v_estado varchar2(6):='ALTA';

  BEGIN
  	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Nombre: '||inmo_nombre||', Ruta: '||inmo_ruta;


  	 --Verifica que el parmetro Cdigo de la imagen no es nulo
	if (inmo_imagen is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;


	-- Actualiza valores de la imagen indicada
	UPDATE IMAGENES
	SET	NOMBRE = NVL(inmo_nombre,NOMBRE),
      RUTA = NVL(inmo_ruta,RUTA)      	 
	WHERE CODIGO_IMAGEN = inmo_imagen;

   RSP:='OK. Modificada de la imagen con codigo: '|| inmo_imagen;	

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la Imagen]INMO_MOD_GENERAL.INMO_MOD_IMAGENES', parametrosIN, RSP,user);	

	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin del inmueble]INMO_MOD_GENERAL.INMO_MOD_INMUEBLES', parametrosIN, error_desc,user);
	  COMMIT;  


END INMO_MOD_IMAGENES;


--- Alta de una actuacin sobre un inmueble
PROCEDURE INMO_ALTA_ACTUACION_INMUEBLE (
                                inmo_tipo_actuacion IN ACTUACIONES_INMUEBLES.CODIGO_TIPO_ACTUACION%TYPE,
				                        inmo_propietario IN ACTUACIONES_INMUEBLES.CODIGO_PROPIETARIO%TYPE,
				                        inmo_interesado IN ACTUACIONES_INMUEBLES.CODIGO_INTERESADO%TYPE,
                                inmo_empleado IN ACTUACIONES_INMUEBLES.CODIGO_EMPLEADO%TYPE,
                                inmo_fecha_alta IN ACTUACIONES_INMUEBLES.FECHA_ALTA%TYPE,
                                inmo_fecha_fin IN ACTUACIONES_INMUEBLES.FECHA_FINALIZACION%TYPE,
                                inmo_situacion IN ACTUACIONES_INMUEBLES.SITUACION%TYPE,
                                inmo_inmueble IN ACTUACIONES_INMUEBLES.cod_inmueble%TYPE,
                                inmo_importe IN ACTUACIONES_INMUEBLES.importe_actuacion%TYPE,
                                inmo_observaciones IN ACTUACIONES_INMUEBLES.OBSERVACIONES%TYPE,
                                RSP OUT varchar2,
                                inmo_cod_actuacion OUT ACTUACIONES_INMUEBLES.CODIGO_ACTUACION%TYPE

) AS

num_inmueble number:=0;
num_propietario number:=0;
num_empleado number:=0;
num_tipo number:=0;
error_desc varchar2(1000); -- descripcion del error
e_error exception; --error controlado
parametrosIN varchar2(1000);  
fecha DATE;
texto TIPO_ACTUACION_INMUEBLE.DESCRIPCION%TYPE;
sede SEDES.CODIGO_SEDE%TYPE;
num_sede number:=0;
v_sede EMPLEADOS_SEDES.CODIGO_SEDE%TYPE;

BEGIN
--Verifica que existe el tipo de actuacin, el propietario,el cdigo de empleado y el inmueble que son los campos obligatorios
BEGIN
 SELECT COUNT(codigo_tipo_actuacion) INTO num_tipo FROM TIPO_ACTUACION_INMUEBLE WHERE
    codigo_tipo_actuacion= inmo_tipo_actuacion;
END;
IF (num_tipo = 0) THEN error_desc:='ERROR: El tipo de actuacin indicado no existe'; RAISE e_error; END IF;

BEGIN
 SELECT COUNT(codigo_cliente) INTO num_propietario FROM CLIENTES WHERE
    codigo_cliente= inmo_propietario;
END;
IF (num_propietario = 0) THEN error_desc:='ERROR: El propietario indicado no existe'; RAISE e_error; END IF;

BEGIN
 SELECT COUNT(codigo_empleado) INTO num_empleado FROM EMPLEADOS WHERE
    codigo_empleado= inmo_empleado;
END;
IF (num_empleado = 0) THEN error_desc:='ERROR: El empleado indicado no existe'; RAISE e_error; END IF;

BEGIN
 SELECT COUNT(codigo_inmueble) INTO num_inmueble FROM INMUEBLES WHERE
    codigo_inmueble= inmo_inmueble;
END;
IF (num_inmueble = 0) THEN error_desc:='ERROR: El inmueble indicado no existe'; RAISE e_error; END IF;

 -- Rellenamos la fecha de alta
 select sysdate into fecha FROM DUAL;


-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Tipo Actuacin: '||inmo_tipo_actuacion||', Propietario: '||inmo_propietario ||', Interesado: '||inmo_interesado ||', Empleado: '||inmo_empleado
                  ||', Fecha de Alta: '||fecha||', Fecha de Finalizacion: '||inmo_fecha_fin ||', Situacin: '||inmo_situacion||', Inmueble: '||inmo_inmueble
                  ||', Importe Actuacin: '||inmo_importe ||', Observaciones: '||inmo_observaciones;


-- Se da de alta la actuacin
 INSERT INTO ACTUACIONES_INMUEBLES 
 VALUES ( TO_CHAR(actuacion_seq.nextval),inmo_tipo_actuacion,inmo_propietario,inmo_interesado,inmo_empleado,inmo_fecha_alta,inmo_fecha_fin,inmo_situacion,inmo_inmueble,inmo_importe,inmo_observaciones);

-- GENERAMOS LOS VALORES ESTADISTICOS
 BEGIN
 SELECT descripcion INTO texto from TIPO_ACTUACION_INMUEBLE
     WHERE codigo_tipo_actuacion = inmo_tipo_actuacion;

 IF (texto = 'ALQUILER') and (inmo_fecha_fin IS NULL) THEN   
 --    Inmueble para alquiler (Consulta1)
  INMO_MOD_ESTADISTICAS.ALTA_DWH_INMUEBLES_ALQUILAR ((EXTRACT(YEAR FROM inmo_fecha_alta)));
 END IF; 

 

 IF ( texto= 'REGIMEN DE VISITAS') THEN
 -- Actuacin de tipo VISITAS. (Consulta 5)
 INMO_MOD_ESTADISTICAS.ALTA_DWH_TRANSACCIONES_VISITAS(EXTRACT(YEAR FROM inmo_fecha_alta),NULL,inmo_empleado);

 END IF;
 
 END;
 
 IF (inmo_importe <> 0 ) THEN
 -- Facturacin de la empresa a un cliente (Consulta 13)
     IF (inmo_interesado IS NOT NULL) THEN
        --- Actuaciones de tipo venta o alquiler u otro tipo donde interviene el interesado.
        INMO_MOD_ESTADISTICAS.ALTA_DWH_FACTURACION_CLIENTE(inmo_interesado,inmo_importe);
     ELSE   
        INMO_MOD_ESTADISTICAS.ALTA_DWH_FACTURACION_CLIENTE (inmo_propietario,inmo_importe);
     END IF;
 END IF;
 
  IF ((texto = 'ALQUILER') OR (texto='VENTA') OR (texto ='COMPRA')) AND (inmo_fecha_fin IS NULL) THEN
 -- Consulta 14. Inmuebles disponibles en el momento de realizar la consulta. Para ello la fecha de finalizacin 
 -- de la actuacin tiene que estar vacia.
 -- Localizamos la sede donde trabaja el empleado que realiza la actuacin.
     SELECT CODIGO_SEDE into v_sede FROM EMPLEADOS_SEDES WHERE CODIGO_EMPLEADO = inmo_empleado and FECHA_BAJA IS NULL;
     INMO_MOD_ESTADISTICAS.ALTA_DWH_INMUEBLES_DISPONIBLES (v_sede);
 END IF;
 
 
 ---FIN DE GENERACION DE ESTADISTICAS

  RSP := 'OK';
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta ACTUACIONES INMUEBLES]INMO_MOD_GENERAL.INMO_ALTA_ACTUACIONES_INMUEBLES', parametrosIN, RSP,user);
  COMMIT;


-- Tratamiento de excepciones

EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta ACTUACIONES INMUEBLES]INMO_MOD_GENERAL.INMO_ALTA_ACTUACIONES_INMUEBLES', parametrosIN, error_desc,user);
      COMMIT;

END INMO_ALTA_ACTUACION_INMUEBLE;


--Baja de una actuacin sobre un inmueble
PROCEDURE INMO_BAJA_ACTUACION_INMUEBLE (inmo_actuacion IN ACTUACIONES_INMUEBLES.CODIGO_ACTUACION%TYPE,inmo_fecha_fin IN ACTUACIONES_INMUEBLES.FECHA_FINALIZACION%TYPE, RSP OUT VARCHAR2) AS
num_codigo number:=0;
error_desc varchar2(1000); -- descripcion del error
e_error exception; --error controlado
parametrosIN varchar2(1000);  
fecha_alta DATE;
texto TIPO_ACTUACION_INMUEBLE.DESCRIPCION%TYPE;
v_sede EMPLEADOS_SEDES.CODIGO_SEDE%TYPE;
v_empleado EMPLEADOS.CODIGO_EMPLEADO%TYPE;
cod_act_pro number;
v_inmueble ACTUACIONES_INMUEBLES.COD_INMUEBLE%TYPE;
v_pais SEDES.PAIS%TYPE;
v_tipoinmueble INMUEBLES.TIPO_INMUEBLE%TYPE;
num_sede NUMBER;
sede VARCHAR2(20);



BEGIN
 --Verifica que la actuacin existe
 BEGIN
 SELECT COUNT(codigo_actuacion) INTO num_codigo FROM ACTUACIONES_INMUEBLES WHERE
    codigo_actuacion= inmo_actuacion;
 END;   

IF (num_codigo = 0) THEN error_desc:='ERROR: La actuacin indicada no existe'; RAISE e_error; END IF;
-- VERIFICA que hay una fecha de finalizacin
IF (inmo_fecha_fin is null ) THEN error_desc:= 'ERROR: La baja requiere una fecha de finalizacin';RAISE e_error; END IF;


-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Actuacin Inmueble: '||inmo_actuacion||', Fecha Finalizacion: '||inmo_fecha_fin;

 --La baja de la actuacin no implica eliminar el registro, sino asignarle una fecha de finalizacin
 UPDATE ACTUACIONES_INMUEBLES  SET FECHA_FINALIZACION = inmo_fecha_fin WHERE CODIGO_ACTUACION= inmo_actuacion;

 
 -- GENERACION DE ESTADISTICAS
 
 SELECT DESCRIPCION INTO texto FROM TIPO_ACTUACION_INMUEBLE 
    WHERE CODIGO_TIPO_ACTUACION = ( SELECT codigo_tipo_actuacion FROM ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_actuacion );
 
 IF (texto = 'ALQUILER') OR (texto='VENTA') THEN
 -- Actuacin de tipo transaccion. (Consulta 4)
 -- localizamos la sede y el empleado de la actuacin
  SELECT CODIGO_EMPLEADO INTO v_empleado FROM ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_actuacion;
  SELECT COUNT(codigo_sede) INTO num_sede FROM EMPLEADOS_SEDES WHERE
    codigo_empleado= v_empleado AND FECHA_BAJA IS NULL ;

   IF (num_sede = 0) THEN error_desc:='ERROR: El empleado no est asignado a ninguna sede el ao en curso .No se puede actualizar Consulta4'; RAISE e_error; END IF;

   -- Empleado de la Sede que esta de alta
   SELECT CODIGO_SEDE into sede from EMPLEADOS_SEDES WHERE (CODIGO_EMPLEADO = v_empleado) AND FECHA_BAJA IS NULL;

   INMO_MOD_ESTADISTICAS.ALTA_DWH_TRANSACCIONES_VISITAS(EXTRACT(YEAR FROM inmo_fecha_fin),sede,v_empleado);

 END IF;
 
 
 -- Comprobamos  el tipo de actuacin .Si tiene fecha finalizacin es que ha sido alquilado (Consulta1)
  IF (texto = 'ALQUILER') THEN   
 --    Inmueble alquilado
   INMO_MOD_ESTADISTICAS.ALTA_DWH_INMUEBLES_ALQUILADOS ((EXTRACT(YEAR FROM inmo_fecha_fin)));
 

 -- Consulta 10:Tipo de inmueble que ms se ha alquilado en cada uno de los pases en los que la empresa tiene sede. 
 
    -- Localizamos el tipo de inmueble de esta actuacin.
     select cod_inmueble into v_inmueble from ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_actuacion;
     SELECT tipo_inmueble into v_tipoinmueble FROM INMUEBLES WHERE CODIGO_INMUEBLE = v_inmueble;
 
     -- Localizamos el empleado de esta actuacin
    select codigo_empleado into v_empleado from ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_actuacion;
 
    -- Localizamos el pais.
    select pais into v_pais FROM SEDES WHERE CODIGO_SEDE = (SELECT CODIGO_SEDE FROM EMPLEADOS_SEDES WHERE CODIGO_EMPLEADO = v_empleado 
              AND FECHA_BAJA IS NULL);
  
     INMO_MOD_ESTADISTICAS.ALTA_DWH_TIPOINMUEBLE_PAIS(v_tipoinmueble,v_pais);
  

 END IF;    

 --- Consulta 7 y Consulta 9
  IF (texto = 'ALQUILER') OR (texto='VENTA') THEN   
 --    Inmueble est alquilado o vendido 
 -- Localiza el empleado que realiza la actuacin
 SELECT codigo_empleado INTO v_empleado FROM ACTUACIONES_INMUEBLES where CODIGO_ACTUACION = inmo_actuacion;
 
 -- Localiza la sede actual del empleado que realiza la actuacin
  SELECT CODIGO_SEDE into v_sede FROM EMPLEADOS_SEDES WHERE  CODIGO_EMPLEADO = v_empleado  AND FECHA_BAJA IS NULL;
  INMO_MOD_ESTADISTICAS.ALTA_DWH_PORCENTAJE_TRANS ((EXTRACT(YEAR FROM inmo_fecha_fin)),v_sede);
    

 END IF;    


   IF ( texto = 'VENTA' ) THEN
 -- Actuacin de tipo venta (Consulta 8)
 -- En el alta de la venta de un inmueble no puede haber gestiones posteriores a la venta. El inmueble est vendido cuando hay fecha de finalizacin
 -- Comprobamos si hay gestiones posteriores a la venta . Hay actuaciones de proveedores posteriores, >= ,que pueden no estar finalizadas por eso comparamos con la fecha de alta.
    SELECT count(codigo_actuacion_pro) into cod_act_pro FROM ACTUACIONES_PROVEEDORES WHERE  TO_DATE(FECHA_ALTA_ACTUACION,'DD/MM/YY') >= TO_DATE(inmo_fecha_fin,'DD/MM/YY' )
          AND (CODIGO_ACTUACION_INMO = inmo_actuacion);
    
      INMO_MOD_ESTADISTICAS.ALTA_DWH_PORCENTAJE_POSVENTA(EXTRACT(YEAR FROM inmo_fecha_fin),cod_act_pro);
    END IF;
-- FIN DE LAS ESTADISTICAS


  RSP := 'OK';
 --Se registra accin en la tabla LOG
 INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja de la Actuacion sobre un inmueble]INMO_MOD_GENERAL.INMO_BAJA_ACTUACIONES_INMUEBLES', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
 WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja de actuaciones sobre un inmueble]INMO_MOD_GENERAL.INMO_BAJA_ACTUACIONES_INMUEBLES', parametrosIN, error_desc,user);
  COMMIT;  

END INMO_BAJA_ACTUACION_INMUEBLE;


-- Modificacin de una actuacin
PROCEDURE INMO_MOD_ACTUACION_INMUEBLE (inmo_cod_actuacion IN ACTUACIONES_INMUEBLES.CODIGO_ACTUACION%TYPE,
                                inmo_tipo_actuacion IN ACTUACIONES_INMUEBLES.CODIGO_TIPO_ACTUACION%TYPE,
				                        inmo_propietario IN ACTUACIONES_INMUEBLES.CODIGO_PROPIETARIO%TYPE,
				                        inmo_interesado IN ACTUACIONES_INMUEBLES.CODIGO_INTERESADO%TYPE,
                                inmo_empleado IN ACTUACIONES_INMUEBLES.CODIGO_EMPLEADO%TYPE,
                                inmo_situacion IN ACTUACIONES_INMUEBLES.SITUACION%TYPE,
                                inmo_inmueble IN ACTUACIONES_INMUEBLES.cod_inmueble%TYPE,
                                inmo_importe IN ACTUACIONES_INMUEBLES.importe_actuacion%TYPE,
                                inmo_observaciones IN ACTUACIONES_INMUEBLES.OBSERVACIONES%TYPE,
                                RSP OUT varchar2 )as
 error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);  
  num_codigo number:=0;
  texto TIPO_ACTUACION_INMUEBLE.DESCRIPCION%TYPE;
  texto_new TIPO_ACTUACION_INMUEBLE.DESCRIPCION%TYPE;
  old_importe ACTUACIONES_INMUEBLES.importe_actuacion%TYPE;
  fecha_fin DATE;
  v_sede EMPLEADOS_SEDES.CODIGO_SEDE%TYPE;
 

BEGIN
 --Verifica que la actuacin existe
 BEGIN
 SELECT COUNT(codigo_actuacion) INTO num_codigo FROM ACTUACIONES_INMUEBLES WHERE
    codigo_actuacion= inmo_cod_actuacion;
 END;   

IF (num_codigo = 0) THEN error_desc:='ERROR: La actuacin indicada no existe'; RAISE e_error; END IF;

-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Actuacin: '||inmo_cod_actuacion||'Tipo Actuacin: '||inmo_tipo_actuacion||', Propietario: '||inmo_propietario ||', Interesado: '||inmo_interesado ||', Empleado: '||inmo_empleado
                  ||', Situacin: '||inmo_situacion||', Inmueble: '||inmo_inmueble
                  ||', Importe Actuacin: '||inmo_importe ||', Observaciones: '||inmo_observaciones;


 	 --Verifica que el parmetro Cdigo de la actuacion no es nulo
	if (inmo_cod_actuacion is NULL) then 
	 error_desc:='ERROR: Parmetro cdigo de la actuacin obligatorio con valor nulo';
	 raise e_error;
	end if;

 -- GENERACION DE ESTADSTICAS SI EL IMPORTE VA A SER MODIFICADO
    IF (inmo_importe <> 0 ) THEN
   -- Facturacin de la empresa a un cliente (Consulta 13)
   -- Localizamos el importe anterior a la modificacin, para restarlo de la facturacin ya que va a ser modificado 
    SELECT IMPORTE_ACTUACION INTO old_importe FROM ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION= inmo_cod_actuacion;
    
     IF (inmo_interesado IS NOT NULL) THEN
        --- Actuaciones de tipo venta o alquiler u otro tipo donde interviene el interesado.
        INMO_MOD_ESTADISTICAS.ALTA_DWH_FACTURACION_CLIENTE(inmo_interesado,inmo_importe-old_importe);
     ELSE   
        INMO_MOD_ESTADISTICAS.ALTA_DWH_FACTURACION_CLIENTE (inmo_propietario,inmo_importe-old_importe);
     END IF;
 END IF;

 -- Comprobamos el tipo de actuacin anterior que no sea Venta, Compra o Alquiler antes de realizar la modificacin
  SELECT descripcion into texto FROM TIPO_ACTUACION_INMUEBLE  WHERE CODIGO_TIPO_ACTUACION = (
           SELECT CODIGO_TIPO_ACTUACION FROM ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_cod_actuacion);
           
 -- Comprobamos que el inmueble sigue disponible. Para ello la fecha fin tiene que estar vacia.         
  SELECT  FECHA_FINALIZACION into fecha_fin FROM ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_cod_actuacion;
  
  IF ((texto <> 'VENTA') OR (texto <> 'COMPRA') OR ( texto <> 'ALQUILER')) AND (fecha_fin is null ) THEN   
     SELECT descripcion into texto_new FROM TIPO_ACTUACION_INMUEBLE where CODIGO_TIPO_ACTUACION = inmo_tipo_actuacion;
     IF (texto_new = 'VENTA') OR (texto_new = 'COMPRA') OR (texto_new = 'ALQUILER') THEN
        -- Localiza la sede del empleado que dio de alta la actuacin.
        SELECT CODIGO_SEDE into v_sede FROM EMPLEADOS_SEDES WHERE CODIGO_EMPLEADO = ( SELECT CODIGO_EMPLEADO FROM ACTUACIONES_INMUEBLES WHERE
            CODIGO_ACTUACION = inmo_cod_actuacion );
     
        INMO_MOD_ESTADISTICAS.ALTA_DWH_INMUEBLES_DISPONIBLES (v_sede);
     END IF;
  END IF;


	-- Actualiza valores de la actuacin indicada
	UPDATE ACTUACIONES_INMUEBLES
	SET	CODIGO_TIPO_ACTUACION = NVL(inmo_tipo_actuacion,CODIGO_TIPO_ACTUACION),
      CODIGO_PROPIETARIO = NVL(inmo_propietario,CODIGO_PROPIETARIO),
      CODIGO_INTERESADO = NVL(inmo_interesado, CODIGO_INTERESADO),
      CODIGO_EMPLEADO = NVL(inmo_empleado, CODIGO_EMPLEADO),
      SITUACION = NVL(inmo_situacion, SITUACION),
      COD_INMUEBLE = NVL(inmo_inmueble, COD_INMUEBLE),
      IMPORTE_ACTUACION = NVL(inmo_importe, IMPORTE_ACTUACION),
      OBSERVACIONES = NVL(inmo_observaciones, OBSERVACIONES)
	WHERE CODIGO_ACTUACION = inmo_cod_actuacion;

   RSP:='OK. Modificada de la actuacin con codigo: '|| inmo_cod_actuacion;	
  
    

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la Actuacin]INMO_MOD_GENERAL.INMO_MOD_ACTUACIONES_INMUEBLES', parametrosIN, RSP,user);	

	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin de LA ACTUACIN DEL INMUEBLE]INMO_MOD_GENERAL.INMO_MOD_ACTUACIONES_INMUEBLES', parametrosIN, error_desc,user);
	  COMMIT;  




END INMO_MOD_ACTUACION_INMUEBLE;


END INMO_MOD_GENERAL;