create or replace PACKAGE INMO_MOD_PROPUESTAS AS 
/*Este paquete contiene los procedimientos y funciomes necesarios de las tablas correspondientes al mantenimiento
del Mdulo Propuestas:  TABLA PROPUESTAS */


/*Descripcin:  	Alta Nuevo Propuesta

Parmetros de entrada:          
				inmo_empleado      cdigo del empleado que realiza la propuesta
        inmo_tipo_propuesta   cdigo del tipo de propuesta
				inmo_fecha          fecha de alta de la propuesta
        inmo_actuacion       cdigo de la actuacin inmobiliaria sobre la que afecta la propuesta
        estado              estado de  la propuesta
        observaciones       observaciones sobre la propuesta

Parmetros de salida
				RSP
				En caso de insercin con xito retorna el cdigo de la propuesta
                                Si la propuesta existe dispara una excpcin


Tareas del procedimiento
        Se verifica que la propuesta la realiza un empleado de categora 'DIRECTOR COMERCIAL'
        Se comprueban que no son nulos los campos obligatorios
				Se da de alta la propuesta
				Se registra accin en la tabla LOG
				Devuelve Si todo ha ido bien:
					RSP = 'OK: Alta de la propuesta con su cdigo
				Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: La propuesta ya existe en el sistema' */


PROCEDURE INMO_ALTA_PROPUESTA (
                                inmo_empleado IN PROPUESTAS.CODIGO_EMPLEADO%TYPE,
				                       inmo_tipo IN PROPUESTAS.CODIGO_TIPO_PROPUESTA%TYPE,
				                        inmo_fecha IN PROPUESTAS.FECHA_PROPUESTA%TYPE,
                                inmo_actuacion IN PROPUESTAS.CODIGO_ACTUACION_INM%TYPE,
                                inmo_estado IN PROPUESTAS.ESTADO%TYPE,  
                                inmo_observaciones IN PROPUESTAS.OBSERVACIONES%TYPE,
                                inmo_importe in PROPUESTAS.IMPORTE%TYPE,
				                        RSP OUT varchar2,
                                inmo_propuesta OUT PROPUESTAS.CODIGO_PROPUESTA%TYPE

);


/*Descripcin: 		Baja de una propuesta

	Parmetro entrada:
						inmo_cod_propuesta: codigo de la propuesta
	Parmetros de salida:
						RSP
						En caso de baja con xito retorna el cdigo de la propuesta
	Tareas
					        Se verifica que la propuesta existe
                  Se da de baja la propuesta.No implica eliminar el registro, sino cambiar su estado a 'BAJA'
						      Se registra accin en la tabla LOG
						Devuelve 
							Si todo ha ido bien:
								RSP = 'OK: Baja propuesta con el cdigo de la propuesta
							Mensajes de error: RSP = 'ERROR: si no existe el cdigo de la propuesta'*/

PROCEDURE INMO_BAJA_PROPUESTA (inmo_cod_propuesta IN PROPUESTAS.CODIGO_PROPUESTA%TYPE, RSP OUT VARCHAR2);



/*Descripcin      modificacin de  campos de la propuesta

Parmetro entrada:      inmo_cod_propuesta cdigo de la propuesta de la que se realiza la modificacin
						inmo_empleado cdigo del empleado
				                inmo_tipo tipo de propuesta
				                inmo_actuacion cdigo de la actuacin inmobiliaria
                        inmo_observaciones observaciones de la propuesta
 Parmetro de salida				RSP	
					       En caso de modificacin con exito retorna el codigo de la propuesta y campo modificado

Tareas						Se modifica el campo de la propuesta que no tenga valor null en el parmetro de entrada correspondiente
                  Se registra la accin en la tabla LOG
                 Devuelve
                      si todo ha ido bien RSP='OK: Modificacin de la propuesta con el cdigo y campo modificado	
						     Mensaje de error : RSP=' ERROR: No existe la propuesta ' or RSP= 'ERROR :Valor de campo invlido'				


*/	
PROCEDURE INMO_MOD_PROPUESTA ( inmo_cod_propuesta IN PROPUESTAS.CODIGO_PROPUESTA%TYPE,
						                  inmo_empleado IN PROPUESTAS.CODIGO_EMPLEADO%TYPE,
				                      inmo_tipo IN PROPUESTAS.CODIGO_TIPO_PROPUESTA%TYPE,
				                      inmo_actuacion IN PROPUESTAS.CODIGO_ACTUACION_INM%TYPE,
                                               inmo_estado IN PROPUESTAS.ESTADO%TYPE,
                               inmo_observaciones IN PROPUESTAS.OBSERVACIONES%TYPE,
                                              						RSP OUT varchar2);



END INMO_MOD_PROPUESTAS;