create or replace PACKAGE BODY INMO_MOD_PROPUESTAS AS

-- Alta de una Propuesta (ACTUACIONES SOBRE EL INMUEBLE OFRECIDAS AL CLIENTE, PERO NO OBLIGATORIAS EN UNA TRANSACCION)

  PROCEDURE INMO_ALTA_PROPUESTA (
                                inmo_empleado IN PROPUESTAS.CODIGO_EMPLEADO%TYPE,
				                        inmo_tipo IN PROPUESTAS.CODIGO_TIPO_PROPUESTA%TYPE,
				                        inmo_fecha IN PROPUESTAS.FECHA_PROPUESTA%TYPE,
                                inmo_actuacion IN PROPUESTAS.CODIGO_ACTUACION_INM%TYPE,
                                inmo_estado IN PROPUESTAS.ESTADO%TYPE,  
                                inmo_observaciones IN PROPUESTAS.OBSERVACIONES%TYPE,
                                inmo_importe in PROPUESTAS.IMPORTE%TYPE,
				                        RSP OUT varchar2,
                                inmo_propuesta OUT PROPUESTAS.CODIGO_PROPUESTA%TYPE

) AS

  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  
  anio VARCHAR2(4);
  v_estado PROPUESTAS.ESTADO%TYPE:='PRESENTADA';
  texto PUESTOS_TRABAJO.DESCRIPCION%TYPE;


  BEGIN

      -- Se comprueba que no son nulos los campos obligatorios
    if (inmo_empleado is NULL or inmo_tipo is NULL or inmo_actuacion is NULL or inmo_fecha is NULL) then 
	      error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	    raise e_error;
	  end if;


    -- Se verifica que la propuesta la realiza un empleado de categora 'DIRECTOR COMERCIAL' en el ao en curso
    BEGIN
         SELECT DESCRIPCION INTO texto FROM PUESTOS_TRABAJO WHERE CODIGO_PUESTO = (SELECT CODIGO_PUESTO_TRABAJO from EMPLEADOS_SEDES WHERE
            CODIGO_EMPLEADO = inmo_empleado  AND FECHA_BAJA IS NULL);
        IF texto <> 'DIRECTOR COMERCIAL' THEN error_desc:='ERROR: El EMPLEADO: ' || inmo_empleado ||' NO PUEDE HACER UNA PROPUESTA. NO ES UN DIRECTOR COMERCIAL'; RAISE e_error; END IF;    
    END;

     -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'EMPLEADO: '||inmo_empleado||', TIPO DE PROPUESTA: '||inmo_tipo||', FECHA DE LA PROPUESTA: '||inmo_fecha||', ACTUACION INMOBILIARIA: '|| inmo_actuacion||',IMPORTE: '|| inmo_importe||', Estado: '||v_estado;

  -- damos de alta la propuesta
  INSERT INTO PROPUESTAS (CODIGO_PROPUESTA,CODIGO_EMPLEADO,CODIGO_TIPO_PROPUESTA,FECHA_PROPUESTA,CODIGO_ACTUACION_INM,OBSERVACIONES,IMPORTE,ESTADO) values
                    ('PTA/000'||TO_CHAR(propuesta_seq.nextval),inmo_empleado,inmo_tipo,inmo_fecha,inmo_actuacion,inmo_observaciones,inmo_importe,v_estado);

   RSP := 'OK';
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta propuesta]INMO_MOD_PROPUESTAS.INMO_ALTA_PROPUESTA', parametrosIN, RSP,user);

  --Generacin de la estadstica (Consulta 2)
  INMO_MOD_ESTADISTICAS.ALTA_DWH_NUM_PROPUESTAS(inmo_fecha,inmo_empleado);

  -- fIN Estadsticas

  COMMIT;
    -- Tratamiento de excepciones

EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Propuesta]INMO_MOD_PROPUESTAS.INMO_ALTA_PROPUESTA ', parametrosIN, error_desc,user);
      COMMIT;                  

  END INMO_ALTA_PROPUESTA;

-- Baja de una propuesta

  PROCEDURE INMO_BAJA_PROPUESTA (inmo_cod_propuesta IN PROPUESTAS.CODIGO_PROPUESTA%TYPE, RSP OUT VARCHAR2) AS
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  num_propuesta number:=0;
  v_estado varchar2(6):='BAJA';

  BEGIN
    --  Se verifica que la propuesta existe
    BEGIN
     SELECT COUNT(codigo_propuesta) INTO num_propuesta FROM PROPUESTAS where CODIGO_PROPUESTA=inmo_cod_propuesta;
     If (num_propuesta = 0) then  error_desc:='ERROR: El LA PROPUESTA: ' || inmo_cod_propuesta ||' NO EXISTE.'; RAISE e_error; END IF;        
    END;

    -- Se da de baja la propuesta cambiando su estado
    UPDATE PROPUESTAS SET ESTADO= v_estado WHERE CODIGO_PROPUESTA= inmo_cod_propuesta;

    RSP := 'OK';

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	   VALUES(log_seq.nextval, SYSDATE,'([Baja de la Propuesta]INMO_MOD_PROPUESTAS.INMO_BAJA_PROPUESTA', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja de la Propuesta]INMO_MOD_PROPUESTAS.INMO_BAJA_PROPUESTAS', parametrosIN, error_desc,user);
  COMMIT;  


  END INMO_BAJA_PROPUESTA;


  -- Modificacin de la Propuesta

  PROCEDURE INMO_MOD_PROPUESTA ( inmo_cod_propuesta IN PROPUESTAS.CODIGO_PROPUESTA%TYPE,
						                  inmo_empleado IN PROPUESTAS.CODIGO_EMPLEADO%TYPE,
				                      inmo_tipo IN PROPUESTAS.CODIGO_TIPO_PROPUESTA%TYPE,
				                      inmo_actuacion IN PROPUESTAS.CODIGO_ACTUACION_INM%TYPE,
                              inmo_estado IN PROPUESTAS.ESTADO%TYPE,
                               inmo_observaciones IN PROPUESTAS.OBSERVACIONES%TYPE,
                                              						RSP OUT varchar2) AS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  fecha DATE;
  v_empleado VARCHAR2(20);
  v_importe NUMBER;


  BEGIN
    	-- Concatenamos los parametros de entrada para el LOG, el estado slo se modifica en el proceso de Baja.
	parametrosIN := 'Cdigo propuesta: '||inmo_cod_propuesta||', Empleado: '||inmo_empleado||', Tipo: '||inmo_tipo||', Actuacin: '||inmo_actuacion||', Estado: '||inmo_estado||', Observaciones: '||inmo_observaciones;

	 --Verifica que el parmetro Cdigo de la Propuesta no es nulo
	if (inmo_cod_propuesta is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;


	-- Actualiza valores de la propuesta indicada
	UPDATE PROPUESTAS
	SET	CODIGO_EMPLEADO = NVL(inmo_empleado,CODIGO_EMPLEADO),
		  CODIGO_TIPO_PROPUESTA = NVL(inmo_tipo,CODIGO_TIPO_PROPUESTA),
		  CODIGO_ACTUACION_INM = NVL(inmo_actuacion,CODIGO_ACTUACION_INM),
      ESTADO = NVL(inmo_estado,ESTADO),
		  OBSERVACIONES = NVL(inmo_observaciones,OBSERVACIONES)

	WHERE CODIGO_PROPUESTA = inmo_cod_propuesta;

  -- Generacin de estadsticas 
  ---Consulta2:Actualizando las propuestas aceptadas.
  IF inmo_estado='ACEPTADA' THEN
    SELECT FECHA_PROPUESTA into fecha from PROPUESTAS WHERE CODIGO_PROPUESTA = inmo_cod_propuesta;
    SELECT CODIGO_EMPLEADO into v_empleado from PROPUESTAS where CODIGO_PROPUESTA = inmo_cod_propuesta;
    INMO_MOD_ESTADISTICAS.ALTA_DWH_PROPUESTAS_ACEPTADAS (fecha,v_empleado);
  --Consulta3 : Actualizamos el Importe de las Propuestas Aceptadas.  
    SELECT IMPORTE INTO v_importe FROM PROPUESTAS WHERE CODIGO_PROPUESTA = inmo_cod_propuesta;
    INMO_MOD_ESTADISTICAS.ALTA_DWH_IMPORTE_PROPUESTAS(extract( year from fecha),v_importe);
  END IF ;


	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la Propuesta]INMO_MOD_PROPUESTAS.INMO_MOD_PROPUESTA', parametrosIN, 'OK',user);	
	RSP:='OK';
	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la Propuesta]INMO_MOD_GENERAL.INMO_MOD_PROPUESTA', parametrosIN, error_desc,user);
	  COMMIT;  


  END INMO_MOD_PROPUESTA;

END INMO_MOD_PROPUESTAS;