create or replace PACKAGE BODY INMO_MOD_PROVEEDORES AS

  PROCEDURE INMO_ALTA_PROVEEDOR (
                                inmo_nombre IN PROVEEDORES.NOMBRE%TYPE,
				                        inmo_direccion IN PROVEEDORES.direccion%TYPE,
                                inmo_cif IN PROVEEDORES.CIF%TYPE,
                                inmo_telefono IN PROVEEDORES.TELEFONO%TYPE,
                                inmo_email IN PROVEEDORES.EMAIL%TYPE,
                                RSP OUT varchar2,
                                inmo_cod_proveedor OUT PROVEEDORES.CODIGO_PROVEEDOR%TYPE
				
) AS
   parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
   anio VARCHAR2(4);
  v_estado VARCHAR2(6):='ALTA';
  codigo_nuevo_proveedor PROVEEDORES.CODIGO_PROVEEDOR%TYPE;


  BEGIN
    --Verifica que no tienen valor nulo los campos obligatorios
    IF (inmo_nombre is null) or (inmo_direccion is null) or (inmo_cif is null) then
       error_desc:='ERROR: Parmetro obligatorio con valor nulo';
      raise e_error;
    END IF;  

    -- Concatenamos los parmetros para el Log
    parametrosIN:= 'Nombre: '||inmo_nombre||'Direccin: '||inmo_direccion||', CIF: '||inmo_cif ||', Telfono: '||inmo_telefono ||', Email: '||inmo_email
                  ||', Estado: '||v_estado;

    -- Aadimos nuevo PROVEEDOR
   INSERT INTO PROVEEDORES (codigo_proveedor,nombre,direccion,cif,telefono,email,estado)
     VALUES ('PROV000/'||TO_CHAR(proveedores_seq.nextval),inmo_nombre,inmo_direccion,inmo_cif,inmo_telefono,inmo_email,v_estado);

  codigo_nuevo_proveedor:='PROV000/'||TO_CHAR(proveedores_seq.currval);   
  RSP := 'OK';
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta PROVEEDOR]INMO_MOD_PROVEEDORES.INMO_ALTA_PROVEEDOR:'|| codigo_nuevo_proveedor , parametrosIN, RSP,user);

  COMMIT;

  -- Tratamiento de excepciones

EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Inmueble]INMO_MOD_PROVEEDORES.INMO_ALTA_PROVEEDOR: '|| codigo_nuevo_proveedor, parametrosIN, error_desc,user);
      COMMIT;


  END INMO_ALTA_PROVEEDOR;

  PROCEDURE INMO_BAJA_PROVEEDOR (inmo_cod_proveedor IN PROVEEDORES.CODIGO_PROVEEDOR%TYPE, RSP OUT VARCHAR2) AS

    error_desc varchar2(1000); -- descripcion del error
    e_error exception; --error controlado
    parametrosIN varchar2(1000);
    num_proveedor number:=0;
    v_estado varchar2(6) := 'BAJA';

BEGIN
-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Proveedor: '||inmo_cod_proveedor;

   -- Verificamos que el proveedor existe
   BEGIN
    SELECT 	count(codigo_proveedor)
    INTO	num_proveedor
    FROM	PROVEEDORES
    WHERE CODIGO_PROVEEDOR = inmo_cod_proveedor;
  END;
  IF (num_proveedor = 0) THEN error_desc:='ERROR: El proveedor indicado para dar de baja no existe'; RAISE e_error; END IF;

  --- Actualizamos el registro con estado BAJA.
  UPDATE PROVEEDORES set ESTADO = v_estado WHERE CODIGO_PROVEEDOR=inmo_cod_proveedor;
  RSP := 'OK';

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja del Proveedor]INMO_MOD_PROVEEDOR.INMO_BAJA_PROVEEDOR: '|| inmo_cod_proveedor, parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja del proveedor]INMO_MOD_PROVEEDOR.INMO_BAJA_PROVEEDOR: '|| inmo_cod_proveedor, parametrosIN, error_desc,user);
  COMMIT;  

  END INMO_BAJA_PROVEEDOR;

  PROCEDURE INMO_MOD_PROVEEDOR (  
                                inmo_cod_proveedor IN PROVEEDORES.CODIGO_PROVEEDOR%TYPE,
                                inmo_nombre IN PROVEEDORES.NOMBRE%TYPE,
				                        inmo_direccion IN PROVEEDORES.direccion%TYPE,
                                inmo_cif IN PROVEEDORES.CIF%TYPE,
                                inmo_telefono IN PROVEEDORES.TELEFONO%TYPE,
                                inmo_email IN PROVEEDORES.EMAIL%TYPE,
				                        RSP OUT varchar2
						) AS

  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);  
  num_codigo number:=0;

  BEGIN
  	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Nombre: '||inmo_nombre||', Direccin: '||inmo_direccion ||',Cif :'|| inmo_cif ||',Telfono: '||inmo_telefono||', Email: '||inmo_email;


  	 --Verifica que el parmetro Cdigo del proveedor no es nulo
	if (inmo_cod_proveedor is NULL) then 
	 error_desc:='ERROR: Parmetro CDIGO del Proveedor obligatorio para la Modificacin de sus valores';
	 raise e_error;
	end if;


	-- Actualiza valores del proveedor indicado
	UPDATE PROVEEDORES
	SET	NOMBRE = NVL(inmo_nombre,NOMBRE),
      DIRECCION = NVL(inmo_direccion,DIRECCION),
      CIF = NVL(inmo_cif,CIF),
      TELEFONO = NVL(inmo_telefono,TELEFONO),
      EMAIL = NVL(inmo_email, EMAIL)
	WHERE CODIGO_PROVEEDOR = inmo_cod_proveedor;

   RSP:='OK. Modificado el proveedor  con codigo: '|| inmo_cod_proveedor;	

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin del Proveedor]INMO_MOD_PROVEEDORES.INMO_MOD_PROVEEDOR: '||inmo_cod_proveedor, parametrosIN, RSP,user);	

	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin del proveedor]INMO_MOD_PROVEEDORES.INMO_MOD_PROVEEDOR: '||inmo_cod_proveedor, parametrosIN, error_desc,user);
	  COMMIT;  

  END INMO_MOD_PROVEEDOR;

-- Alta Actuacin de un proveedor
  PROCEDURE INMO_ALTA_ACTUACION_PROVEEDOR ( inmo_tipo_actuacion IN ACTUACIONES_PROVEEDORES.CODIGO_ACTUACION_PRO%TYPE,
                                          inmo_proveedor IN ACTUACIONES_PROVEEDORES.CODIGO_PROVEEDOR%TYPE,
                                          inmo_empleado IN ACTUACIONES_PROVEEDORES.CODIGO_EMPLEADO%TYPE,
                                          inmo_fecha_alta IN ACTUACIONES_PROVEEDORES.FECHA_ALTA_ACTUACION%TYPE,
                                          inmo_fecha_finalizacion IN ACTUACIONES_PROVEEDORES.SITUACION%TYPE,
                                          inmo_situacion IN ACTUACIONES_PROVEEDORES.SITUACION%TYPE,
                                          inmo_actuacion_inmo IN ACTUACIONES_PROVEEDORES.CODIGO_ACTUACION_INMO%TYPE,                                          
                                          inmo_observaciones IN ACTUACIONES_PROVEEDORES.OBSERVACIONES%TYPE,
                                          RSP OUT VARCHAR2,
                                          inmo_actuacion_proveedor out ACTUACIONES_PROVEEDORES.CODIGO_ACTUACION_INMO%TYPE) AS
num_proveedor number:=0;
num_empleado number:=0;
num_act_inmo number:=0;
num_tipo number:=0;
error_desc varchar2(1000); -- descripcion del error
e_error exception; --error controlado
parametrosIN varchar2(1000);  
fecha DATE;
texto PUESTOS_TRABAJO.DESCRIPCION%TYPE;
des_actuacion TIPO_ACTUACION_PROVEEDOR.DESCRIPCION%TYPE;
v_cliente VARCHAR2(20);

BEGIN
--Verifica que existe el proveedor,el tipo de actuacion, el empleado y la actuacin inmobiliaria a la que afecta esta actuacin.
BEGIN
 SELECT COUNT(codigo_tipo_act_proV) INTO num_tipo FROM TIPO_ACTUACION_PROVEEDOR WHERE
    codigo_tipo_act_PROV= inmo_tipo_actuacion;
END;
IF (num_tipo = 0) THEN error_desc:='ERROR: El tipo de actuacin indicado  para AlTA ACTUACIONES PROVEEDOR no existe'; RAISE e_error; END IF;

BEGIN
 SELECT COUNT(codigo_proveedor) INTO num_proveedor FROM PROVEEDORES WHERE
    codigo_proveedor= inmo_proveedor;
END;
IF (num_proveedor = 0) THEN error_desc:='ERROR: El proveedor indicado no existe para ALTA ACTUACIONES PROVEEDOR'; RAISE e_error; END IF;

BEGIN
 SELECT COUNT(codigo_empleado) INTO num_empleado FROM EMPLEADOS WHERE
    codigo_empleado= inmo_empleado;
END;
IF (num_empleado = 0) THEN error_desc:='ERROR: El empleado indicado no existe'; RAISE e_error;

ELSE
 -- chequa que el tipo de empleado que realiza la actuacin es 'DIRECTOR COMERCIAL'
SELECT DESCRIPCION into texto FROM PUESTOS_TRABAJO 
    WHERE   CODIGO_PUESTO = ( SELECT CODIGO_PUESTO_TRABAJO FROM EMPLEADOS_SEDES WHERE CODIGO_EMPLEADO= inmo_empleado and FECHA_BAJA IS NULL);

 IF texto <> 'DIRECTOR COMERCIAL' THEN   error_desc:='ERROR: El Empleado que interviene en la Actuacin con el Proveedor tiene que ser DIRECTOR COMERCIAL '; RAISE e_error; END IF;
END IF;

BEGIN
 SELECT COUNT(codigo_actuacion) INTO num_act_inmo FROM ACTUACIONES_INMUEBLES WHERE
    codigo_actuacion= inmo_actuacion_inmo;
END;

IF (num_act_inmo = 0) THEN error_desc:='ERROR: La Actuacin Inmobiliaria no existe para esta Actuacin del Proveedor'; RAISE e_error; END IF;

 -- Rellenamos la fecha de alta
 select sysdate into fecha FROM DUAL;


-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Tipo Actuacin: '||inmo_tipo_actuacion||', Proveedor: '||inmo_proveedor ||', Empleado: '||inmo_empleado ||', Situacin: '||inmo_situacion 
                  ||', Actuacin Inmobiliaria: '||inmo_actuacion_inmo ||', Observaciones: '||inmo_observaciones;


-- Se da de alta la actuacin
 INSERT INTO ACTUACIONES_PROVEEDORES (codigo_actuacion_pro,codigo_tipo_actuacion_pro,codigo_proveedor,codigo_empleado,fecha_alta_actuacion,situacion,codigo_actuacion_inmo,observaciones)
 VALUES ( 'ACTPROV000/'||TO_CHAR(actuacionpro_seq.nextval),inmo_tipo_actuacion,inmo_proveedor,inmo_empleado,inmo_fecha_alta,inmo_situacion,inmo_actuacion_inmo
 ,inmo_observaciones);

-- GENERAMOS LOS VALORES ESTADISTICOS
-- Consulta 6: Porcentaje de incremento respecto al mismo mes del ao anterior de los servicios solicitados a proveedores
INMO_MOD_ESTADISTICAS.ALTA_DWH_PORCENTAJE_PROVE(Extract(year from inmo_fecha_alta),Extract(month from inmo_fecha_alta));


--Consulta 12: cliente que ms litigios jurdicos ha presentado contra la empresa'
IF inmo_tipo_actuacion ='TAPRV0001' THEN
   SELECT CODIGO_PROPIETARIO  INTO v_cliente FROM  ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_actuacion_inmo;
   INMO_MOD_ESTADISTICAS.ALTA_DWH_LITIGIOS_JURIDICOS (v_cliente);
END IF;   

-- FIN DE GENERACION DE ESTADISTICAS
 
  RSP := 'OK';
 
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta ACTUACIONES PROVEEDORES]INMO_MOD_GENERAL.INMO_ALTA_ACTUACIONES_PROVEEDOR', parametrosIN, RSP,user);
  COMMIT;


-- Tratamiento de excepciones
commit;
EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta ACTUACION PROVEEDOR]INMO_MOD_PROVEEDORES.INMO_ALTA_ACTUACION_PROVEEDOR:' ||inmo_actuacion_proveedor, parametrosIN, error_desc,user);
      COMMIT;

  END INMO_ALTA_ACTUACION_PROVEEDOR;

  PROCEDURE INMO_BAJA_ACTUACION_PROVEEDOR (inmo_actuacion_proveedor IN ACTUACIONES_PROVEEDORES.CODIGO_PROVEEDOR%TYPE, RSP OUT VARCHAR2) AS

  num_codigo number:=0;
error_desc varchar2(1000); -- descripcion del error
e_error exception; --error controlado
parametrosIN varchar2(1000);  
fecha DATE;

BEGIN
 --Verifica que la actuacin existe
 BEGIN
 SELECT COUNT(codigo_actuacion_pro) INTO num_codigo FROM ACTUACIONES_PROVEEDORES WHERE
    codigo_actuacion_pro= inmo_actuacion_proveedor;
 END;   

IF (num_codigo = 0) THEN error_desc:='ERROR: La actuacin indicada no existe para darla de BAJA'; RAISE e_error; END IF;


-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Actuacin Inmueble: '||inmo_actuacion_proveedor;

 --La baja de la actuacin no implica eliminar el registro, sino asignarle una fecha de finalizacin
 UPDATE ACTUACIONES_PROVEEDORES  SET FECHA_FINALIZACION = SYSDATE WHERE CODIGO_ACTUACION_PRO= inmo_actuacion_proveedor;


 --Se registra accin en la tabla LOG
 INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja de la Actuacion de un Proveedor]INMO_MOD_GENERAL.INMO_BAJA_ACTUACIONES_PROVEEDORES: '|| inmo_actuacion_proveedor , parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja de la Actuacion de un Proveedor]INMO_MOD_GENERAL.INMO_BAJA_ACTUACIONES_PROVEEDORES: '|| inmo_actuacion_proveedor, parametrosIN, error_desc,user);
  COMMIT;  

  END INMO_BAJA_ACTUACION_PROVEEDOR;



  PROCEDURE INMO_MOD_ACTUACION_PROVEEDOR (  inmo_actuacion_proveedor IN  ACTUACIONES_PROVEEDORES.CODIGO_ACTUACION_PRO%TYPE,
                                          inmo_tipo_actuacion IN ACTUACIONES_PROVEEDORES.CODIGO_ACTUACION_PRO%TYPE,
                                          inmo_proveedor IN ACTUACIONES_PROVEEDORES.CODIGO_PROVEEDOR%TYPE,
                                          inmo_empleado IN ACTUACIONES_PROVEEDORES.CODIGO_EMPLEADO%TYPE,
                                          inmo_situacion IN ACTUACIONES_PROVEEDORES.SITUACION%TYPE,
                                          inmo_actuacion_inmo IN ACTUACIONES_PROVEEDORES.CODIGO_ACTUACION_INMO%TYPE,
                                          inmo_observaciones IN ACTUACIONES_PROVEEDORES.OBSERVACIONES%TYPE,
                                          RSP OUT VARCHAR2 ) AS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);  
  num_codigo number:=0;
  num_tipo number:=0;
  num_act_inmueble number:=0;
  num_empleado number:=0;
  texto TIPO_ACTUACION_INMUEBLE.DESCRIPCION%TYPE;
  v_sede EMPLEADOS_SEDES.CODIGO_SEDE%TYPE;

BEGIN
 --Verifica que la actuacin existe
 BEGIN
 SELECT COUNT(codigo_actuacion_pro) INTO num_codigo FROM ACTUACIONES_PROVEEDORES WHERE
    codigo_actuacion_pro= inmo_actuacion_proveedor;
 END;   

IF (num_codigo = 0) THEN error_desc:='ERROR: La actuacin del proveedor no existe '||inmo_actuacion_proveedor; RAISE e_error; END IF;

-- Verifica que el tipo de actuacin del proveedor existe

BEGIN
 SELECT COUNT(CODIGO_TIPO_ACT_PROV) INTO num_tipo FROM TIPO_ACTUACION_PROVEEDOR
        WHERE CODIGO_TIPO_ACT_PROV=inmo_tipo_actuacion;
END;
IF (num_tipo = 0) THEN error_desc:='ERROR: El tipo de actuacin del proveedor no existe '||inmo_tipo_actuacion; RAISE e_error; END IF;

-- Verifica que la actuacin del inmueble al que hace referencia existe.
BEGIN
  SELECT COUNT(CODIGO_ACTUACION) into num_act_inmueble FROM ACTUACIONES_INMUEBLES 
        WHERE CODIGO_ACTUACION = inmo_actuacion_inmo;
END;
IF (num_act_inmueble = 0) THEN error_desc:='ERROR: La actuacin del inmueble asociada a la actuacin del proveedor no existe '||inmo_actuacion_inmo; RAISE e_error; END IF;


-- Verifica que el empleado existe
BEGIN
  SELECT COUNT(CODIGO_EMPLEADO) INTO num_empleado FROM EMPLEADOS 
      WHERE CODIGO_EMPLEADO=inmo_empleado;

END;
IF (num_empleado =0) THEN error_desc:='ERROR: El empleado asociado a la actuacin del proveedor no existe '||inmo_empleado; RAISE e_error; END IF;

	 --Verifica que el parmetro Cdigo de la actuacion no es nulo
	if (inmo_actuacion_proveedor is NULL) then 
	 error_desc:='ERROR: Parmetro cdigo de la actuacin obligatorio con valor nulo para MODIFICACION ACTUACION PROVEEDOR';
	 raise e_error;
	end if;

-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Actuacin: '||inmo_actuacion_proveedor||'Tipo Actuacin: '||inmo_tipo_actuacion||', Proveedor: '||inmo_proveedor ||', Empleado: '||inmo_empleado
                  ||', Situacin: '||inmo_situacion||', Inmueble: '||inmo_actuacion_inmo  ||', Observaciones: '||inmo_observaciones;



	-- Actualiza valores de la actuacin indicada
	UPDATE ACTUACIONES_PROVEEDORES
	SET	CODIGO_TIPO_ACTUACION_PRO = NVL(inmo_tipo_actuacion,CODIGO_TIPO_ACTUACION_PRO),
      CODIGO_PROVEEDOR = NVL(inmo_proveedor,CODIGO_PROVEEDOR),
      CODIGO_EMPLEADO = NVL(inmo_empleado, CODIGO_EMPLEADO),
      SITUACION = NVL(inmo_situacion, SITUACION),
      CODIGO_ACTUACION_INMO = NVL(inmo_actuacion_inmo, CODIGO_ACTUACION_INMO),
      OBSERVACIONES = NVL(inmo_observaciones, OBSERVACIONES)
	WHERE CODIGO_ACTUACION_PRO = inmo_actuacion_proveedor;

   RSP:='OK. Modificada de la actuacin con codigo: '|| inmo_actuacion_proveedor;	

/* --Se generan las estadsticas. 
 -- Comprobamos si el tipo de actuacin es ALQUILER O VENTA
 SELECT DESCRIPCION INTO texto FROM TIPO_ACTUACION_INMUEBLE 
    WHERE CODIGO_TIPO_ACTUACION = ( SELECT codigo_tipo_actuacion FROM ACTUACIONES_INMUEBLES WHERE CODIGO_ACTUACION = inmo_cod_actuacion );

 IF (texto = 'ALQUILER') OR (texto='VENTA') THEN   
 --    Inmueble en alquiler o venta.
 -- Localiza la sede actual del empleado que realiza la actuacin
  SELECT CODIGO_SEDE into v_sede FROM EMPLEADOS_SEDES WHERE  CODIGO_EMPLEADO = inmo_empleado AND EXTRACT(YEAR FROM FECHA_ALTA) <= EXTRACT (YEAR FROM SYSDATE) 
     AND FECHA_BAJA IS NULL;
  INMO_MOD_ESTADISTICAS.ALTA_DWH_PORCENTAJE_TRANS ((EXTRACT(YEAR FROM SYSDATE)),v_sede);

 END IF;    
*/
	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la Actuacin]INMO_MOD_GENERAL.INMO_MOD_ACTUACION_PROVEEDOR: '|| inmo_actuacion_proveedor, parametrosIN, RSP,user);	

	COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin de la Actuacin]INMO_MOD_GENERAL.INMO_MOD_ACTUACION_PROVEEDOR: '|| inmo_actuacion_proveedor, parametrosIN, error_desc,user);
	  COMMIT;  


  END INMO_MOD_ACTUACION_PROVEEDOR;

END INMO_MOD_PROVEEDORES;