create or replace PACKAGE INMO_MOD_INTERNO AS 
/*Este paquete contiene los procedimientos y funciomes necesarios de las tablas correspondientes al mantenimiento
del Mdulo Interno: EMPLEADOS, EMPLEADOS_SEDES*/


/*Descripcin:  	Alta Nuevo Empleado

Parmetros de entrada:          
                                inmo_nombre             nombre del empleado
                                inmo_direccion          direccin del empleado
                                inmo_dni                documento de identidad del empleado
                                inmo_fecha              fecha de nacimiento del empleado
                                inmo_email              email profesional del empleado

Parmetros de salida
				RSP
				En caso de insercin con xito retorna el cdigo del empleado,
                                Si el empleado existe dispara una excpcin


Tareas del procedimiento
				Se da de alta el empleado
				Se registra accin en la tabla LOG
				Devuelve Si todo ha ido bien:
					RSP = 'OK'
				Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El empleado ya existe en el sistema' */


PROCEDURE INMO_ALTA_EMPLEADOS (
                                inmo_nombre IN EMPLEADOS.nombre%TYPE,
                                inmo_direccion IN EMPLEADOS.direccion%TYPE,
                                inmo_dni IN EMPLEADOS.DOCUMENTO_IDENTIDAD%TYPE,
                                inmo_fecha IN EMPLEADOS.FECHA_NACIMIENTO%TYPE,
                                inmo_email IN EMPLEADOS.EMAIL_PROFESIONAL%TYPE,                                
				                        RSP OUT varchar2,
                                inmo_empleado OUT EMPLEADOS.codigo_empleado%TYPE

);


/*Descripcin: 		Baja de un empleado

	Parmetro entrada:
						inmo_empleado: codigo del empleado
	Parmetros de salida:
						RSP
						En caso de baja con xito retorna el cdigo del empleado
	Tareas
					        Se da de baja el cliente
                  Se registra accin en la tabla LOG
						Devuelve 
							Si todo ha ido bien:
								RSP = 'OK: Baja cliente con el cdigo del empleado
							Mensajes de error: RSP = 'ERROR: si no existe el cdigo de empleado'*/

PROCEDURE INMO_BAJA_EMPLEADOS (inmo_empleado IN EMPLEADOS.CODIGO_EMPLEADO%TYPE, RSP OUT VARCHAR2);


/*Descripcin      modificacin de un empleado

Parmetro entrada:                              inmo_empleado cdigo del empleado del que se realiza la modificacin
                                                inmo_nombre nombre del empleado
                                                inmo_direccion direccin del empleado
                                                inmo_dni documento de identidad del empleado
                                                inmo_fecha fecha de nacimiento del empleado
                                                inmo_email email profesional del empleado

Parmetro de salida				RSP	
                        En caso de modificacin con exito retorna el codigo del empleado y campo modificado

Tareas						                              Se modifica el campo del empleado que no tenga valor null en el parmetro de entrada correspondiente.La fecha de alta no se modifica
                                                Se registra la accin en la tabla LOG
                                                Devuelve
                                                 si todo ha ido bien RSP='OK: Modificacin del empleado con el cdigo y campo modificado	
                                                Mensaje de error : RSP=' ERROR: No existe el empleado ' or RSP= 'ERROR :Valor de campo invlido'				


*/	
PROCEDURE INMO_MOD_EMPLEADOS ( inmo_empleado IN EMPLEADOS.codigo_empleado%TYPE,
                              inmo_nombre IN EMPLEADOS.nombre%TYPE,
                              inmo_direccion IN EMPLEADOS.direccion%TYPE,
                              inmo_fecha IN EMPLEADOS.FECHA_NACIMIENTO%TYPE,
                              inmo_dni IN EMPLEADOS.DOCUMENTO_IDENTIDAD%TYPE,
                               inmo_email IN EMPLEADOS.EMAIL_PROFESIONAL%TYPE,	
						RSP OUT varchar2);


/*Descripcin:  	Alta de un empleado en una sede

Parmetros de entrada:          inmo_empleado           cdigo del empleado
                                inmo_sede               cdigo de la sede
                                inmo_puesto             cdigo del puesto de trabajo del empleado en la sede
                                inmo_fecha_alta         fecha de alta del empleado en la sede
                                inmo_jornada            jornada de trabajo del empleado en la sede
                                inmo_superior           superior jerrquico del empleado en la sede				
Parmetros de salida
				RSP
				En caso de insercin con xito retorna el cdigo del empleado y el cdigo de la sedde
                                Si el empleado existe en esa sede dispara una excepcin


Tareas del procedimiento
				Se da de alta el empleado en la sede
				Se registra accin en la tabla LOG
				Devuelve Si todo ha ido bien:
					RSP = 'OK'
				Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El empleado ya existe en esa sede dentro del sistema' */


PROCEDURE INMO_ALTA_EMPLEADOS_SEDES (
                                inmo_empleado IN EMPLEADOS_SEDES.codigo_empleado%TYPE,
                                inmo_sede IN EMPLEADOS_SEDES.CODIGO_SEDE%TYPE,
                                inmo_puesto IN EMPLEADOS_SEDES.CODIGO_PUESTO_TRABAJO%TYPE,
                                inmo_fecha IN EMPLEADOS_SEDES.FECHA_ALTA%TYPE,
                                inmo_jornada IN EMPLEADOS_SEDES.JORNADA%TYPE,
                                inmo_superior IN EMPLEADOS_SEDES.SUPERIOR_JERARQUICO%TYPE,                                
				                        RSP OUT varchar2
     );


/*Descripcin: 		Baja de un empleado en una sede

	Parmetro entrada:
						inmo_empleado: codigo del empleado
            inmo_sede: cdigo de la sede
	Parmetros de salida:
						RSP
						En caso de baja con xito retorna el cdigo del empleado y de la sede
	Tareas
					        Se da de baja el empleado en la sede. En este caso se elimina el registro. No se queda incongruencia en los datos.
                  Se registra accin en la tabla LOG
						Devuelve 
							Si todo ha ido bien:
								RSP = 'OK: Baja del empleado en la sede con el cdigo del empleado y cdigo de la sede
							Mensajes de error: RSP = 'ERROR: si no existe el cdigo de empleado o de la sede o ambos'*/

PROCEDURE INMO_BAJA_EMPLEADOS_SEDES (inmo_empleado IN EMPLEADOS_SEDES.CODIGO_EMPLEADO%TYPE,
                                     inmo_sede IN EMPLEADOS_SEDES.CODIGO_SEDE%TYPE,
                                      RSP OUT VARCHAR2);


/*Descripcin      cambiar a un empleado de sede  modificar la informacin

Parmetro entrada:                              inmo_empleado cdigo del empleado del que se realiza la modificacin 
                                                inmo_sede cdigo de la nueva sede del empleado
                                                inmo_puesto cdigo del nuevo puesto de trabajo del empleado en la sede
                                                inmo_fecha fecha de alta del empleado en la nueva sede
                                                inmo_jornada nueva jornada del empleado en la sede
                                                inmo_superior nuevo superior jerrquico del empleado en la sede

Parmetro de salida				RSP	
                        En caso de modificacin con exito retorna el codigo del empleado, el de la sede y campo modificado

Tareas						                              Se modifica el campo del empleado en la sede que no tenga valor null en el parmetro de entrada correspondiente.La fecha de alta no se modifica
                                                Se registra la accin en la tabla LOG
                                                Devuelve
                                                 si todo ha ido bien RSP='OK: Modificacin del empleado en la sede y campo modificado	
                                                Mensaje de error : RSP=' ERROR: No existe el empleado o la sede ' or RSP= 'ERROR :Valor de campo invlido'				


*/	
PROCEDURE INMO_MOD_EMPLEADOS_SEDES ( inmo_empleado IN EMPLEADOS_SEDES.codigo_empleado%TYPE,
                              inmo_sede IN EMPLEADOS_SEDES.codigo_sede%TYPE,
                              inmo_puesto IN EMPLEADOS_SEDES.codigo_puesto_trabajo%TYPE,
                               inmo_jornada IN EMPLEADOS_SEDES.JORNADA%TYPE,
                              inmo_superior IN EMPLEADOS_SEDES.SUPERIOR_JERARQUICO%TYPE,	
						RSP OUT varchar2);

END INMO_MOD_INTERNO;