create or replace PACKAGE BODY INMO_MOD_INTERNO AS

-- Alta de empleados

  PROCEDURE INMO_ALTA_EMPLEADOS (
                                inmo_nombre IN EMPLEADOS.nombre%TYPE,
                                inmo_direccion IN EMPLEADOS.direccion%TYPE,
                                inmo_dni IN EMPLEADOS.DOCUMENTO_IDENTIDAD%TYPE,
                                inmo_fecha IN EMPLEADOS.FECHA_NACIMIENTO%TYPE,
                                inmo_email IN EMPLEADOS.EMAIL_PROFESIONAL%TYPE,                                
				                        RSP OUT varchar2,
                                inmo_empleado OUT EMPLEADOS.codigo_empleado%TYPE
				
) AS
  
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  COD_NUEVO_EMPLEADO VARCHAR2(20);
  v_estado VARCHAR2(6):='ALTA';
  BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Nombre: '||inmo_nombre||', Direccion: '||inmo_direccion||', DNI: '||inmo_dni||', Fecha de nacimiento  : '|| inmo_fecha||', Email: '||inmo_email||', Estado: '||v_estado;

 --Verifica que todos los campos obligatorios tienen valor
	if (inmo_nombre is NULL or inmo_direccion is NULL or inmo_dni is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

 -- Aadimos nuevo empleado


  INSERT INTO EMPLEADOS
  (codigo_empleado,nombre,direccion,documento_identidad,fecha_nacimiento,email_profesional,estado)   VALUES ('EMP/000'||TO_CHAR(empleados_seq.nextval),inmo_nombre,inmo_direccion,inmo_dni,inmo_fecha,inmo_email,v_estado);
  RSP := 'OK';
    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([ALTA del Empleado]INMO_MOD_INTERNOL.INMO_ALTA_EMPLEADOS', parametrosIN, RSP,user);


  --COD_NUEVO_EMPLEADO := TO_CHAR(empleados_seq.currval);
  COMMIT;


EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Empleado ]INMO_MOD_INTERNO.INMO_ALTA_EMPLEADOS', parametrosIN, error_desc,user);
      COMMIT;



  END INMO_ALTA_EMPLEADOS;


-- Baja de un empleado
  PROCEDURE INMO_BAJA_EMPLEADOS (inmo_empleado IN EMPLEADOS.CODIGO_EMPLEADO%TYPE, RSP OUT VARCHAR2) AS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_empleado number:=0;
  v_BAJA CONSTANT VARCHAR2(6) := 'BAJA'; 
  BEGIN

    -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Empleado: '||inmo_empleado;

   -- Verificamos que el empleado existe
   BEGIN
    SELECT 	count(inmo_empleado)
    INTO	num_empleado
    FROM	EMPLEADOS
    WHERE CODIGO_EMPLEADO = inmo_empleado; 
  END;
  IF (num_empleado = 0) THEN error_desc:='ERROR: El Empleado indicado no existe'; RAISE e_error; END IF;


  -- Los empleados no se eliminan. Se modifica su estado a BAJA
  UPDATE EMPLEADOS SET ESTADO = v_BAJA WHERE CODIGO_EMPLEADO = inmo_empleado;

 	RSP := 'OK';

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja del Empleado]INMO_MOD_INTERNOL.INMO_BAJA_EMPLEADOS', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja del Empleado]INMO_MOD_INTERNO.INMO_BAJA_EMPLEADOS', parametrosIN, error_desc,user);
  COMMIT;  

  END INMO_BAJA_EMPLEADOS;


-- Modificacin de un empleado
  PROCEDURE INMO_MOD_EMPLEADOS ( inmo_empleado IN EMPLEADOS.codigo_empleado%TYPE,
                              inmo_nombre IN EMPLEADOS.nombre%TYPE,
                              inmo_direccion IN EMPLEADOS.direccion%TYPE,
                              inmo_fecha IN EMPLEADOS.FECHA_NACIMIENTO%TYPE,
                              inmo_dni IN EMPLEADOS.DOCUMENTO_IDENTIDAD%TYPE,
                              inmo_email IN EMPLEADOS.EMAIL_PROFESIONAL%TYPE,	
						RSP OUT varchar2) AS

  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);         

  BEGIN
  	-- Concatenamos los parametros de entrada para el LOG, el estado slo se modifica en el proceso de Baja.
	parametrosIN := 'Cdigo Empleado: '||inmo_empleado||', Nombre: '||inmo_nombre||', Direccion: '||inmo_direccion||', DNI: '||inmo_dni||', Fecha de Nacimiento: '||inmo_fecha||', Email Profesional: '||inmo_email;

	 --Verifica que el parmetro Cdigo de Empleado no es nulo
	if (inmo_empleado is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

		-- Actualiza valores del empleado indicado
	UPDATE EMPLEADOS
  	SET	NOMBRE = NVL(inmo_nombre,NOMBRE),
		  DIRECCION = NVL(inmo_direccion,DIRECCION),
		  DOCUMENTO_IDENTIDAD = NVL(inmo_dni,DOCUMENTO_IDENTIDAD),
		  FECHA_NACIMIENTO = NVL(inmo_fecha,FECHA_NACIMIENTO),
		  EMAIL_PROFESIONAL = NVL(inmo_email,EMAIL_PROFESIONAL)
	WHERE CODIGO_EMPLEADO = inmo_empleado;

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	      VALUES(log_seq.nextval, SYSDATE,'([Modificacin del Empleado]INMO_MOD_INTERNO.INMO_MOD_EMPLEADOS', parametrosIN, 'OK',user);	
	RSP:='OK';
  COMMIT;


 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin del Empleado]INMO_MOD_INTERNO.INMO_MOD_EMPLEADOS', parametrosIN, error_desc,user);
	  COMMIT;  


  END INMO_MOD_EMPLEADOS;


  -- Alta de un empleado en una sede

  PROCEDURE INMO_ALTA_EMPLEADOS_SEDES (
                                inmo_empleado IN EMPLEADOS_SEDES.codigo_empleado%TYPE,
                                inmo_sede IN EMPLEADOS_SEDES.CODIGO_SEDE%TYPE,
                                inmo_puesto IN EMPLEADOS_SEDES.CODIGO_PUESTO_TRABAJO%TYPE,
                                inmo_fecha IN EMPLEADOS_SEDES.FECHA_ALTA%TYPE,
                                inmo_jornada IN EMPLEADOS_SEDES.JORNADA%TYPE,
                                inmo_superior IN EMPLEADOS_SEDES.SUPERIOR_JERARQUICO%TYPE,                                
				                        RSP OUT varchar2
     ) AS

     num_empleado integer;
     num_sede integer;
     error_desc varchar2(1000); -- descripcion del error
     e_error exception; --error controlado
     parametrosIN varchar2(1000); 
     fecha date;
     v_empleado integer;
     v_pais SEDES.PAIS%TYPE;

  BEGIN

     -- Comprobamos que el empleado existe
     BEGIN
      SELECT 	count(inmo_empleado)
      INTO	num_empleado
      FROM	EMPLEADOS
      WHERE CODIGO_EMPLEADO = inmo_empleado; 
     END;

  IF (num_empleado = 0) THEN error_desc:='ERROR: El Empleado indicado no existe'; RAISE e_error; END IF; 

     -- Comprobamos que la sede existe
      BEGIN
      SELECT 	count(inmo_sede)
      INTO	num_sede
      FROM	SEDES
      WHERE CODIGO_SEDE = inmo_sede; 
  END;
   IF (num_sede = 0) THEN error_desc:='ERROR: La sede indicada no existe'; RAISE e_error; END IF;

   -- Preparamos el parmetro del LOG
   parametrosIN := 'Cdigo Empleado: '||inmo_empleado||', Sede: '||inmo_sede||', Puesto de Trabajo: '||inmo_puesto||', Fecha de alta: '||inmo_fecha||', Jornada: '||inmo_jornada||', Superior Jerrquico: '||inmo_superior;

    --Verifica que todos los campos obligatorios tienen valor
	if (inmo_puesto is NULL or  inmo_jornada is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

 -- Comprobamos que el empleado no est dado de alta en otra sede
 SELECT COUNT(codigo_empleado) into v_empleado from EMPLEADOS_SEDES WHERE (CODIGO_EMPLEADO = inmo_empleado) AND FECHA_BAJA IS NULL;
 
  IF v_empleado <> 0 THEN
    error_desc:='ERROR: Ee empleado: '|| inmo_empleado || 'ya trabaja en una sede.';
	 raise e_error;
  
  END IF;
 
 -- Aadimos nuevo empleado en la sede
  
  INSERT INTO EMPLEADOS_SEDES
  (codigo_empleado,codigo_sede,codigo_puesto_trabajo,fecha_alta,jornada,superior_jerarquico)   VALUES (inmo_empleado,inmo_sede,inmo_puesto,inmo_fecha,inmo_jornada,inmo_superior);
 
 
  RSP := 'OK';

    -- Actualizamos el log con la accin realizada
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta empleado en la sede]INMO_MOD_INTERNO.INMO_ALTA_EMPLEADOS_SEDES', parametrosIN, RSP,user);

  COMMIT;

  -- ACTUALIZAMOS INFORMACIN ESTADSTICA Consulta 11
  -- Localizamos el pais de la sede donde va a trabajar el empleado.  
  SELECT PAIS INTO v_pais from SEDES where CODIGO_SEDE = inmo_sede;  
  INMO_MOD_ESTADISTICAS.ALTA_DWH_CONTRATADOS_PAISES (v_pais);  
    
  

  -- Tratamiento de excepciones

EXCEPTION  
      WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS(cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta Empleados Sede]INMO_MOD_INTERNO.INMO_ALTA_EMPLEADOS_SEDE', parametrosIN, error_desc,user);
      COMMIT;


  END INMO_ALTA_EMPLEADOS_SEDES;

  PROCEDURE INMO_BAJA_EMPLEADOS_SEDES (inmo_empleado IN EMPLEADOS_SEDES.CODIGO_EMPLEADO%TYPE,
                                     inmo_sede IN EMPLEADOS_SEDES.CODIGO_SEDE%TYPE,
                                      RSP OUT VARCHAR2) AS
 error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_empleado number:=0;
  num_sede number:=0;

  v_BAJA CONSTANT VARCHAR2(6) := 'BAJA'; 
  BEGIN

    -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'Cdigo Empleado: '||inmo_empleado||', Cdigo de la sede: '|| inmo_sede;

   -- Verificamos que el empleado existe
   BEGIN
    SELECT 	count(inmo_empleado)
    INTO	num_empleado
    FROM	EMPLEADOS
    WHERE CODIGO_EMPLEADO = inmo_empleado; 
  END;
  IF (num_empleado = 0) THEN error_desc:='ERROR: El Empleado indicado no existe'; RAISE e_error; END IF;

  --Verificamos que la sede existe
  BEGIN
    SELECT 	count(inmo_sede)
    INTO	num_sede
    FROM	SEDES
    WHERE CODIGO_SEDE = inmo_sede; 
  END;
  IF (num_sede = 0) THEN error_desc:='ERROR: La sedde indicada no existe'; RAISE e_error; END IF;

  -- Se rellena la fecha de baja en tabla EMPLEADOS_SEDE. 
  update EMPLEADOS_SEDES set FECHA_BAJA = SYSDATE WHERE CODIGO_EMPLEADO = inmo_empleado AND CODIGO_SEDE = inmo_sede;

 	RSP := 'OK';

	INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja del Empleado en la sede]INMO_MOD_INTERNO.INMO_BAJA_EMPLEADOS_SEDES', parametrosIN, RSP,user);

	COMMIT;

EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja del Empleado en la sede]INMO_MOD_INTERNO.INMO_BAJA_EMPLEADOS_SEDES', parametrosIN, error_desc,user);
  COMMIT;  
  END INMO_BAJA_EMPLEADOS_SEDES;


 -- Modificamos valores del empleado en la sede o lo cambiamos de sede 

  PROCEDURE INMO_MOD_EMPLEADOS_SEDES ( inmo_empleado IN EMPLEADOS_SEDES.codigo_empleado%TYPE,
                              inmo_sede IN EMPLEADOS_SEDES.codigo_sede%TYPE,
                              inmo_puesto IN EMPLEADOS_SEDES.codigo_puesto_trabajo%TYPE,                             
                              inmo_jornada IN EMPLEADOS_SEDES.JORNADA%TYPE,
                              inmo_superior IN EMPLEADOS_SEDES.SUPERIOR_JERARQUICO%TYPE,	
						RSP OUT varchar2) AS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);         
   num_empleado_sede number:=0; 
   num_empleado number:=0;
   num_sede number:=0;


  BEGIN
  	-- Concatenamos los parametros de entrada para el LOG, el estado slo se modifica en el proceso de Baja.
	parametrosIN := 'Cdigo Empleado: '||inmo_empleado||', Cdigo de la Sede: '||inmo_sede||', Puesto de Trabajo: '||inmo_puesto||', jornada: '||inmo_jornada||', Superior jerrquico: '||inmo_superior;

	 --Verifica que el parmetro Cdigo de Empleado y el cdigo de la sede no son nulos para modificar algn valor
	if (inmo_empleado is NULL) or (inmo_sede is Null) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

 -- Verifica que existe el empleado en la tabla EMPLEADOS
 BEGIN
   SELECT COUNT(codigo_empleado) INTO num_empleado FROM EMPLEADOS 
      where CODIGO_EMPLEADO = inmo_empleado; 
 END;

 IF (num_empleado = 0) THEN error_desc:='ERROR: El empleado indicado no existe'; RAISE e_error; END IF;

 -- Verifica que existe la sede en la tabla SEDES
  BEGIN
   SELECT COUNT(codigo_sede) INTO num_sede FROM SEDES
      where CODIGO_SEDE = inmo_sede; 
 END;

 IF (num_sede = 0) THEN error_desc:='ERROR:La sede indicada no existe'; RAISE e_error; END IF;


 -- Verifica que el empleado existe en esa sede en la tabla EMPLEADOS_SEDES
 BEGIN
 SELECT COUNT(codigo_empleado) into num_empleado_sede FROM EMPLEADOS_SEDES
      WHERE codigo_empleado=inmo_empleado AND codigo_sede = inmo_sede;
  END;

 IF (num_empleado_sede = 0) THEN 
  --- CAMBIAMOS AL EMPLEADO DE SEDE SI ES EL CASO , NO existe el registro en EMPLEADOS_SEDES con el valor de empleado y sede introducido como parmetro.
  DELETE FROM EMPLEADOS_SEDES WHERE CODIGO_EMPLEADO = inmo_empleado AND CODIGO_SEDE = inmo_sede;
  INSERT INTO EMPLEADOS_SEDES(CODIGO_EMPLEADO,CODIGO_SEDE,CODIGO_PUESTO_TRABAJO,FECHA_ALTA,JORNADA,SUPERIOR_JERARQUICO,FECHA_BAJA)
        VALUES (inmo_empleado,inmo_sede,inmo_puesto,SYSDATE,inmo_jornada,inmo_superior,NULL);
  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	      VALUES(log_seq.nextval, SYSDATE,'([Cambio del Empleado a una Nueva Sede]INMO_MOD_INTERNO.INMO_MOD_EMPLEADOS_SEDES', parametrosIN, 'OK',user);	
	RSP:='OK';
  COMMIT;

 ELSE
 -- Modificamos los valores del Empleado en la sede
 UPDATE EMPLEADOS_SEDES
  	SET	CODIGO_PUESTO_TRABAJO = NVL(inmo_puesto,CODIGO_PUESTO_TRABAJO),
		  JORNADA = NVL(inmo_jornada,JORNADA),
		   SUPERIOR_JERARQUICO= NVL(inmo_superior,SUPERIOR_JERARQUICO)
	WHERE CODIGO_EMPLEADO = inmo_empleado AND CODIGO_SEDE= inmo_sede;

  INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	      VALUES(log_seq.nextval, SYSDATE,'([Modificacin de valores del Empleado en la Sede]INMO_MOD_INTERNO.INMO_MOD_EMPLEADOS_SEDES', parametrosIN, 'OK',user);	
	RSP:='OK';
  COMMIT;

 END IF;

 EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INMO_LOGS (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin del Empleado en la Sede]INMO_MOD_INTERNO.INMO_MOD_EMPLEADOS_SEDES', parametrosIN, error_desc,user);
	  COMMIT;  

  END INMO_MOD_EMPLEADOS_SEDES;

END INMO_MOD_INTERNO;